class PositionMode:

    def __init__(self):
        self.dualSidePosition = None

    @staticmethod
    def json_parse(json_data):
        result = PositionMode()
        result.dualSidePosition = json_data.get_boolean("dualSidePosition")

        return result



import time


def watchFuturesLimitTrigger(gate, symbol, orderId, doPutTpSl, cancelIfNotOpened, params):
    if doPutTpSl:
        if 'tpSlOrderSide' not in params.keys() or 'stopLoss' not in params.keys() or 'takeProfit' not in params.keys():
            raise ValueError('Must specify \'tpSlOrderSide\' and \'stopLoss\' and \'takeProfit\'')

    if cancelIfNotOpened:
        if 'cancelDelaySec' not in params.keys():
            raise ValueError('Must specify \'cancelDelaySec\'')
        delayTimeSec = float(params['cancelDelaySec'])
        startDelayTime = time.time()

    print('Watching order')
    while True:
        time.sleep(0.1)
        order = gate.getOrder(symbol=symbol, orderId=orderId, futures=True)

        if cancelIfNotOpened:
            if time.time() - startDelayTime > delayTimeSec:
                gate.cancelOrder(symbol, orderId=orderId, futures=True)
                break

        if order['status'] == 'NEW':
            continue
        elif order['status'] == 'FILLED':
            if doPutTpSl:
                orderSide = params['tpSlOrderSide']
                stopLoss = params['stopLoss']
                takeProfit = params['takeProfit']

                stopLossOrder = gate.createAndTestFuturesOrder(symbol, orderSide, 'STOP_MARKET',
                                                               stopPrice=str(stopLoss), closePosition=True,
                                                               priceProtect=True, workingType='MARK_PRICE',
                                                               timeInForce='GTC')

                takeProfitOrder = gate.createAndTestFuturesOrder(symbol, orderSide, 'TAKE_PROFIT_MARKET',
                                                                 stopPrice=str(takeProfit), closePosition=True,
                                                                 priceProtect=True, workingType='MARK_PRICE',
                                                                 timeInForce='GTC')
                result = gate.makeBatchFuturesOrder([stopLossOrder, takeProfitOrder])
                print(result)
                break
        elif order['status'] == 'CANCELED':
            break

    print('Watching position')
    while True:
        time.sleep(0.1)
        position = gate.getPositionInfo(symbol)[0]

        if float(position['entryPrice']) == 0.0:
            gate.cancelAllSymbolOpenOrders(symbol, futures=True)
            break



from abc import ABC, abstractmethod


class BaseExchange(ABC):
    @abstractmethod
    def __init__(self, credentials, sandbox=False, unifiedInOuts=True):
        pass

    @abstractmethod
    def getBalance(self, asset='', futures=False):
        pass

    @abstractmethod
    def symbolAccountTradeHistory(self, symbol, futures=False, fromId=None, limit=None):
        pass

    @abstractmethod
    def testSpotOrder(self, orderData):
        pass

    @abstractmethod
    def makeSpotOrder(self, orderData):
        pass

    @abstractmethod
    def getSymbolOrders(self, symbol, futures=False, orderId=None, startTime=None, endTime=None, limit=None):
        pass

    @abstractmethod
    def getOpenOrders(self, symbol, futures=False):
        pass

    @abstractmethod
    def cancelAllSymbolOpenOrders(self, symbol, futures=False):
        pass

    @abstractmethod
    def cancelOrder(self, symbol, orderId=None, localOrderId=None, futures=False):
        pass

    @abstractmethod
    def getOrder(self, symbol, orderId=None, localOrderId=None, futures=False):
        pass

    @abstractmethod
    def getTradingFees(self):
        pass

    @abstractmethod
    def getSymbolTickerPrice(self, symbol, futures=False):
        pass

    @abstractmethod
    def getSymbolKlines(self, symbol, interval, startTime=None, endTime=None, limit=None, futures=False, blvtnav=False,
                        convertDateTime=False, doClean=False, toCleanDataframe=False):
        pass

    @abstractmethod
    def getExchangeTime(self, futures=False):
        pass

    @abstractmethod
    def getSymbol24hTicker(self, symbol):
        pass

    @abstractmethod
    def testFuturesOrder(self, futuresOrderData):
        pass

    @abstractmethod
    def makeFuturesOrder(self, futuresOrderData):
        pass

    @abstractmethod
    def makeBatchFuturesOrder(self, futuresOrderDatas):
        pass

    @abstractmethod
    def changeInitialLeverage(self, symbol, leverage):
        pass

    @abstractmethod
    def changeMarginType(self, symbol, marginType, params):
        pass

    @abstractmethod
    def changePositionMargin(self, symbol, amount, marginType=None):
        pass

    @abstractmethod
    def getPosition(self):
        pass

    @abstractmethod
    def spotBestBidAsks(self, symbol=None):
        pass

    @abstractmethod
    def getSymbolOrderBook(self, symbol, limit=None, futures=False):
        pass

    @abstractmethod
    def getSymbolRecentTrades(self, symbol, limit=None, futures=False):
        pass

    @abstractmethod
    def getPositionInfo(self, symbol=None):
        pass

    @abstractmethod
    def getSymbolMinTrade(self, symbol, futures=False):
        pass



import logging
import time

from binance.spot import Spot

from Exchanges.BaseExchange import BaseExchange
from Utils import BinanceHelpers, DataHelpers
from binance_f import RequestClient
from binance_f.model.balance import Balance


def isOrderDataValid(order: DataHelpers.OrderData):
    if order.orderType not in BinanceExchange.spotOrderTypes:
        return False

    if order.side not in ['BUY', 'SELL']:
        return False

    if order.newOrderRespType not in [None, 'ACK', 'RESULT', 'FULL']:
        return False

    if order.timeInForce not in [None, 'GTC', 'IOC', 'FOK']:
        return False

    if order.orderType == 'LIMIT':
        if not (order.timeInForce is None or order.quantity is None or order.price is None):
            return True

    elif order.orderType == 'MARKET':
        if not (order.quantity is None and order.quoteOrderQty is None):
            return True

    elif order.orderType in ['STOP_LOSS', 'TAKE_PROFIT']:
        if not (order.quantity is None or order.stopPrice is None):
            return True

    elif order.orderType in ['STOP_LOSS_LIMIT', 'TAKE_PROFIT_LIMIT']:
        if not (
                order.timeInForce is None or order.quantity is None or order.price is None or order.stopPrice is None):
            return True

    elif order.orderType == 'LIMIT_MAKER':
        if not (order.quantity is None or order.price is None):
            return True

    return False


def isFuturesOrderDataValid(order: DataHelpers.futuresOrderData):
    if order.side not in ['BUY', 'SELL']:
        return False

    if order.orderType not in BinanceExchange.futuresOrderTypes:
        return False

    if order.positionSide not in [None, 'BOTH', 'LONG', 'SHORT']:
        return False

    if order.timeInForce not in [None, 'GTC', 'IOC', 'FOK', 'GTX']:
        return False

    if order.workingType not in [None, 'MARK_PRICE', 'CONTRACT_PRICE']:
        return False

    if order.newOrderRespType not in [None, 'ACK', 'RESULT']:
        return False

    if order.closePosition not in [None, True, False]:
        return False

    if order.callbackRate is not None and not (0.1 <= order.callbackRate <= 5):
        return False

    if order.priceProtect not in [None, True, False]:
        return False

    if order.closePosition is True and order.quantity is not None:
        return False

    if order.reduceOnly not in [None, True, False]:
        return False

    if order.closePosition is True and order.reduceOnly is True:
        return False

    if order.orderType == 'LIMIT':
        if not (order.timeInForce is None or order.quantity is None or order.price is None):
            return True

    elif order.orderType == 'MARKET':
        if order.quantity is not None:
            return True

    elif order.orderType in ['STOP', 'TAKE_PROFIT']:
        if not (order.quantity is None or order.price is None or order.stopPrice is None):
            return True

    elif order.orderType in ['STOP_MARKET', 'TAKE_PROFIT_MARKET']:
        if order.stopPrice is not None:
            return True

    elif order.orderType == 'TRAILING_STOP_MARKET':
        if order.callbackRate is not None:
            return True


class BinanceExchange(BaseExchange):
    timeIntervals = ['1m', '3m', '5m', '15m', '30m', '1h', '2h', '4h', '6h', '8h', '12h', '1d', '3d', '1w',
                     '1M']

    timeIndexesInCandleData = [0, 6]
    desiredCandleDataIndexes = [0, 1, 2, 3, 4, 5, 6, 8]

    spotOrderTypes = ['LIMIT', 'MARKET', 'STOP_LOSS', 'STOP_LOSS_LIMIT', 'TAKE_PROFIT',
                      'TAKE_PROFIT_LIMIT', 'LIMIT_MAKER']

    futuresOrderTypes = ['LIMIT', 'MARKET', 'STOP', 'STOP_MARKET', 'TAKE_PROFIT', 'TAKE_PROFIT_MARKET',
                         'TRAILING_STOP_MARKET']

    def __init__(self, credentials, sandbox=False, unifiedInOuts=True):
        self.credentials = credentials
        self.sandbox = sandbox
        self.unifiedInOuts = unifiedInOuts

        if sandbox:
            self.client = Spot(key=credentials['spot']['key'], secret=credentials['spot']['secret'],
                               base_url='https://testnet.binance.vision')
            self.futuresClient = RequestClient(api_key=credentials['futures']['key'],
                                               secret_key=credentials['futures']['secret'],
                                               url='https://testnet.binancefuture.com')
        else:
            self.client = Spot(key=credentials['spot']['key'], secret=credentials['spot']['secret'])
            self.futuresClient = RequestClient(api_key=credentials['futures']['key'],
                                               secret_key=credentials['futures']['secret'],
                                               url='https://fapi.binance.com')

        self.subFutureClient = None

    def getBalance(self, asset='', futures=False):
        if not futures:
            try:
                balances = self.client.account()['balances']
            except Exception:
                return None

            if asset == '':
                return balances
            else:
                for balance in balances:
                    if balance['asset'] == asset:
                        return balance
            return None
        else:
            balances = []
            for balance in self.futuresClient.get_balance():
                balances.append(balance.toDict())

            if asset == '':
                return balances
            else:
                for balance in balances:
                    if balance['asset'] == asset:
                        return balance
                return Balance.makeFreeBalance(asset)

    def symbolAccountTradeHistory(self, symbol, futures=False, fromId=None, limit=None):
        try:
            if not futures:
                return self.client.my_trades(symbol, fromId=fromId, limit=limit)
            else:
                trades = []
                for trade in self.futuresClient.get_account_trades(symbol=symbol, fromId=fromId, limit=limit):
                    trades.append(trade.toDict())
                return trades

        except Exception:
            return None

    def testSpotOrder(self, orderData):
        if not isOrderDataValid(orderData):
            raise ValueError('Incomplete data provided.')

        orderData.setTimestamp()
        params = BinanceHelpers.getSpotOrderAsDict(orderData)

        response = self.client.new_order_test(**params)
        return response

    def makeSpotOrder(self, orderData):
        params = BinanceHelpers.getSpotOrderAsDict(orderData)

        response = self.client.new_order(**params)
        logging.info(response)
        return response

    def getSymbolOrders(self, symbol, futures=False, orderId=None, startTime=None, endTime=None, limit=None):
        try:
            if not futures:
                return self.client.get_orders(symbol, orderId=orderId, startTime=startTime, endTime=endTime,
                                              limit=limit, timestamp=time.time())
            else:
                orders = []
                for order in self.futuresClient.get_all_orders(symbol, orderId=orderId, startTime=startTime,
                                                               endTime=endTime, limit=limit):
                    orders.append(order.toDict())
                return orders
        except Exception:
            return None

    def getOpenOrders(self, symbol, futures=False):
        try:
            if not futures:
                return self.client.get_open_orders(symbol, timestamp=time.time())
            else:
                orders = []
                for order in self.futuresClient.get_open_orders(symbol=symbol):
                    orders.append(order.toDict())
                return orders
        except Exception:
            return None

    def cancelAllSymbolOpenOrders(self, symbol, futures=False):
        if not futures:
            openOrders = self.getOpenOrders(symbol)
            if len(openOrders) == 0:
                return []
            else:
                return self.client.cancel_open_orders(symbol, timestamp=time.time())
        else:
            openOrders = self.getOpenOrders(symbol, futures=True)

            if len(openOrders) == 0:
                return []
            else:
                orderIds = [order['orderId'] for order in openOrders]

                results = []
                for res in self.futuresClient.cancel_list_orders(symbol=symbol, orderIdList=orderIds):
                    results.append(res.toDict())

                return results

    def cancelOrder(self, symbol, orderId=None, localOrderId=None, futures=False):
        errorMessage = 'Specify either order Id in the exchange or local Id sent with the order'
        if not futures:
            if orderId is not None:
                return self.client.cancel_order(symbol, orderId=orderId, timestamp=time.time())
            elif localOrderId is not None:
                return self.client.cancel_order(symbol, origClientOrderId=localOrderId, timestamp=time.time())
            else:
                raise ValueError(errorMessage)
        else:
            if orderId is not None:
                return self.futuresClient.cancel_order(symbol, orderId=orderId).toDict()
            elif localOrderId is not None:
                return self.futuresClient.cancel_order(symbol, origClientOrderId=localOrderId).toDict()
            else:
                raise ValueError(errorMessage)

    def getOrder(self, symbol, orderId=None, localOrderId=None, futures=False):
        errorMessage = 'Specify either order Id in the exchange or local Id sent with the order'
        if not futures:
            if orderId is not None:
                return self.client.get_order(symbol, orderId=orderId, timestamp=time.time())
            elif localOrderId is not None:
                return self.client.get_order(symbol, origClientOrderId=localOrderId, timestamp=time.time())
            else:
                raise ValueError(errorMessage)
        else:
            if orderId is not None:
                return self.futuresClient.get_order(symbol, orderId=orderId).toDict()
            elif localOrderId is not None:
                return self.futuresClient.get_order(symbol, origClientOrderId=localOrderId).toDict()
            else:
                raise ValueError(errorMessage)

    def getTradingFees(self):
        try:
            return self.client.trade_fee()
        except Exception:
            return None

    def getSymbolTickerPrice(self, symbol, futures=False):
        if futures:
            return self.futuresClient.get_symbol_price_ticker(symbol=symbol)[0].price
        else:
            return float(self.client.ticker_price(symbol)['price'])

    def getSymbolKlines(self, symbol, interval, startTime=None, endTime=None, limit=None, futures=False, blvtnav=False,
                        convertDateTime=False, doClean=False, toCleanDataframe=False):
        if interval not in BinanceExchange.timeIntervals:
            raise ValueError('Time interval is not valid.')

        if futures:
            data = self._getFuturesSymbolKlines(blvtnav, endTime, interval, limit, startTime, symbol)
        else:
            data = self._getSpotSymbolKlines(endTime, interval, limit, startTime, symbol)

        if convertDateTime or toCleanDataframe:
            BinanceHelpers.klinesConvertDate(data, self.timeIndexesInCandleData)

        if doClean or toCleanDataframe:
            finalDataArray = BinanceHelpers.getKlinesDesiredOnlyCols(data, self.desiredCandleDataIndexes)

            if toCleanDataframe:
                return BinanceHelpers.klinesConvertToPandas(finalDataArray)
            return finalDataArray
        else:
            return data

    def _getSpotSymbolKlines(self, endTime, interval, limit, startTime, symbol):
        data = self.client.klines(symbol, interval, startTime=startTime, endTime=endTime, limit=limit)
        for datum in data:
            for idx in range(len(datum)):
                if idx in BinanceExchange.timeIndexesInCandleData:
                    continue
                datum[idx] = float(datum[idx])
        return data

    def _getFuturesSymbolKlines(self, blvtnav, endTime, interval, limit, startTime, symbol):
        data = []
        if blvtnav:
            candles = self.futuresClient.get_blvt_nav_candlestick_data(symbol=symbol, interval=interval,
                                                                       startTime=startTime, endTime=endTime,
                                                                       limit=limit)
        else:
            candles = self.futuresClient.get_candlestick_data(symbol=symbol, interval=interval, startTime=startTime,
                                                              endTime=endTime, limit=limit)
        for candle in candles:
            data.append(candle.toArray())
        return data

    def getExchangeTime(self, futures=False):
        try:
            if not futures:
                return self.client.time()['serverTime']
            else:
                return self.futuresClient.get_servertime()
        except Exception:
            return None

    def getSymbol24hTicker(self, symbol):
        try:
            return self.client.ticker_24hr(symbol)
        except Exception:
            return None

    def testFuturesOrder(self, futuresOrderData):
        if not isFuturesOrderDataValid(futuresOrderData):
            raise ValueError('Incomplete data provided.')
        return futuresOrderData

    def makeFuturesOrder(self, futuresOrderData):
        params = BinanceHelpers.getFuturesOrderAsDict(futuresOrderData)

        response = self.futuresClient.post_order(**params)
        return response.toDict()

    def makeBatchFuturesOrder(self, futuresOrderDatas):
        batchOrders = BinanceHelpers.makeBatchOrderData(futuresOrderDatas)

        orderResults = self.futuresClient.post_batch_order(batchOrders)

        return [order.toDict() for order in orderResults]

    def cancelAllSymbolFuturesOrdersWithCountDown(self, symbol, countdownTime):
        return self.futuresClient.auto_cancel_all_orders(symbol, countdownTime)

    def changeInitialLeverage(self, symbol, leverage):
        return self.futuresClient.change_initial_leverage(symbol=symbol, leverage=leverage).toDict()

    def changeMarginType(self, symbol, marginType, params=None):
        if marginType not in ['ISOLATED', 'CROSSED']:
            raise ValueError('Margin type specified is not acceptable')

        return self.futuresClient.change_margin_type(symbol=symbol, marginType=marginType)

    def changePositionMargin(self, symbol, amount, marginType=None):
        if marginType not in ['ISOLATED', 'CROSSED']:
            raise ValueError('marginType was not correctly specified, should be either ISOLATED or CROSSED')

        return self.futuresClient.change_margin_type(symbol, marginType)

    def getPosition(self):
        return self.futuresClient.get_position()

    def spotBestBidAsks(self, symbol=None):
        return self.client.book_ticker(symbol=symbol)

    def getSymbolOrderBook(self, symbol, limit=None, futures=False):
        if not futures:
            if limit is None:
                return self.client.depth(symbol)
            else:
                return self.client.depth(symbol, limit=limit)
        else:
            if limit is None:
                return self.futuresClient.get_order_book(symbol=symbol)
            else:
                return self.futuresClient.get_order_book(symbol=symbol, limit=limit)

    def getSymbolRecentTrades(self, symbol, limit=None, futures=False):
        if limit is not None:
            if limit > 1000:
                limit = 1000
            elif limit < 1:
                limit = 1
        if not futures:
            if limit is None:
                return self.client.trades(symbol)
            else:
                return self.client.trades(symbol, limit=limit)
        else:
            if limit is None:
                return self.futuresClient.get_recent_trades_list(symbol=symbol)
            else:
                return self.futuresClient.get_recent_trades_list(symbol=symbol, limit=limit)

    def getPositionInfo(self, symbol=None):
        return self.futuresClient.get_position_v2(symbol)

    def getSymbolMinTrade(self, symbol, futures=False):
        tickerPrice = self.getSymbolTickerPrice(symbol, futures=futures)

        if futures:
            exchangeInfo = self.futuresClient.get_exchange_information()

            for sym in exchangeInfo.symbols:
                if sym.symbol == symbol:
                    symbolFilters = sym.filters
                    return BinanceHelpers.extractSymbolInfoFromFilters(symbolFilters, tickerPrice)
            return None
        else:
            exchangeInfo = self.client.exchange_info()

            for sym in exchangeInfo['symbols']:
                if sym['symbol'] == symbol:
                    symbolFilters = sym['filters']
                    return BinanceHelpers.extractSymbolInfoFromFilters(symbolFilters, tickerPrice)
            return None



class Trade:

    def __init__(self):
        self.id = None
        self.price = 0.0
        self.qty = 0.0
        self.quoteQty = 0.0
        self.time = 0
        self.isBuyerMaker = False

    @staticmethod
    def json_parse(json_data):
        result = Trade()
        result.id = json_data.get_int("id")
        result.price = json_data.get_float("price")
        result.qty = json_data.get_float("qty")
        result.quoteQty = json_data.get_float("quoteQty")
        result.time = json_data.get_int("time")
        result.isBuyerMaker = json_data.get_boolean("isBuyerMaker")

        return result

    def toDict(self):
        return {
            'id': self.id,
            'price': self.price,
            'qty': self.qty,
            'quoteQty': self.quoteQty,
            'time': self.time,
            'isBuyerMaker': self.isBuyerMaker,
        }




class BinanceApiException(Exception):

    RUNTIME_ERROR = "RuntimeError"
    INPUT_ERROR = "InputError"
    KEY_MISSING = "KeyMissing"
    SYS_ERROR = "SysError"
    SUBSCRIPTION_ERROR = "SubscriptionError"
    ENV_ERROR = "EnvironmentError"
    EXEC_ERROR = "ExecuteError"

    def __init__(self, error_code, error_message):
        self.error_code = error_code
        self.error_message = error_message



class CountdownCancelAll:

    def __init__(self):
        self.symbol = ""
        self.countdownTime = 0

    @staticmethod
    def json_parse(json_data):
        result = CountdownCancelAll()
        result.symbol = json_data.get_string("symbol")
        result.countdownTime = json_data.get_int("countdownTime")

        return result



from kucoin_futures.marke_data.market_data import MarketData
from kucoin_futures.trade.trade import TradeData
from kucoin_futures.user.user import UserData
from kucoin_futures.ws_token.token import GetToken


class FuturesMarket(MarketData):
    pass


class FuturesUser(UserData):
    pass


class FuturesTrade(TradeData):
    pass


class FuturesWsToken(GetToken):
    pass



import json
from binance_f.impl.utils.jsonwrapper import JsonWrapper


def parse_json_from_string(value):
    value = value.replace("False","false")
    value = value.replace("True","true")
    return JsonWrapper(json.loads(value))



class Position:

    def __init__(self):
        self.entryPrice = 0.0
        self.isAutoAddMargin = False
        self.leverage = 0.0
        self.maxNotionalValue = 0.0
        self.liquidationPrice = 0.0
        self.markPrice = 0.0
        self.positionAmt = 0.0
        self.symbol = ""
        self.unrealizedProfit = 0.0
        self.marginType = ""
        self.isolatedMargin = 0.0
        self.positionSide = ""

    @staticmethod
    def json_parse(json_data):
        result = Position()
        result.entryPrice = json_data.get_float("entryPrice")
        result.isAutoAddMargin = json_data.get_boolean("isAutoAddMargin")
        result.leverage = json_data.get_float("leverage")
        result.maxNotionalValue = json_data.get_float("maxNotionalValue")
        result.liquidationPrice = json_data.get_float("liquidationPrice")
        result.markPrice = json_data.get_float("markPrice")
        result.positionAmt = json_data.get_float("positionAmt")
        result.symbol = json_data.get_string("symbol")
        result.unrealizedProfit = json_data.get_float("unRealizedProfit")
        result.marginType = json_data.get_string("marginType")
        result.isolatedMargin = json_data.get_float("isolatedMargin")
        result.positionSide = json_data.get_string("positionSide")
        return result

    def toDict(self):
        return {
            'entryPrice': self.entryPrice,
            'isAutoAddMargin': self.isAutoAddMargin,
            'leverage': self.leverage,
            'maxNotionalValue': self.maxNotionalValue,
            'liquidationPrice': self.liquidationPrice,
            'markPrice': self.markPrice,
            'positionAmt': self.positionAmt,
            'symbol': self.symbol,
            'unrealizedProfit': self.unrealizedProfit,
            'marginType': self.marginType,
            'isolatedMargin': self.isolatedMargin,
            'positionSide': self.positionSide,
        }



import multiprocessing

from Exchanges import BinanceExchange, BybitExchange
from Utils import DataHelpers
from Watchers.futureOrderWatchers import watchFuturesLimitTrigger
from binance_f.exception.binanceapiexception import BinanceApiException


class TradeGate:
    def __init__(self, configDict, sandbox=False):
        self.exchangeName = configDict['exchangeName']
        exchangeClass = self.getCorrectExchange(self.exchangeName)
        if sandbox:
            self.apiKey = configDict['credentials']['test']['spot']['key']
            self.apiSecret = configDict['credentials']['test']['spot']['secret']

            self.exchange = exchangeClass(configDict['credentials']['test'], sandbox=True)
        else:
            self.apiKey = configDict['credentials']['main']['spot']['key']
            self.apiSecret = configDict['credentials']['main']['spot']['secret']

            self.exchange = exchangeClass(configDict['credentials']['main'], sandbox=False)

    def getBalance(self, asset='', futures=False):
        return self.exchange.getBalance(asset, futures)

    @staticmethod
    def getCorrectExchange(exchangeName):
        if exchangeName.lower() == 'binance':
            return BinanceExchange.BinanceExchange
        if exchangeName.lower() == 'bybit':
            return BybitExchange.BybitExchange

    @staticmethod
    def setSpotOrderData(icebergQty, newClientOrderId, newOrderRespType, orderType, price, quantity, recvWindow, side,
                         stopPrice, symbol, timeInForce):
        currOrder = DataHelpers.OrderData(symbol.upper(), side.upper(), orderType.upper())
        if quantity is not None:
            currOrder.setQuantity(quantity)
        if price is not None:
            currOrder.setPrice(price)
        if timeInForce is not None:
            currOrder.setTimeInForce(timeInForce)
        if stopPrice is not None:
            currOrder.setStopPrice(stopPrice)
        if icebergQty is not None:
            currOrder.setIcebergQty(icebergQty)
        if newOrderRespType is not None:
            currOrder.setNewOrderRespType(newOrderRespType)
        if recvWindow is not None:
            currOrder.setRecvWindow(recvWindow)
        if newClientOrderId is not None:
            currOrder.setNewClientOrderId(newClientOrderId)
        return currOrder

    @staticmethod
    def setFuturesOrderData(activationPrice, callbackRate, closePosition, extraParams, newClientOrderId,
                            newOrderRespType, orderType, positionSide, price, priceProtect, quantity, recvWindow,
                            reduceOnly, side, stopPrice, symbol, timeInForce, workingType):
        if extraParams is None:
            extraParams = {}
        currOrder = DataHelpers.futuresOrderData(symbol.upper(), side.upper(), orderType.upper())
        if positionSide is not None:
            currOrder.setPositionSide(positionSide)
        if timeInForce is not None:
            currOrder.setTimeInForce(timeInForce)
        if quantity is not None:
            currOrder.setQuantity(quantity)
        if reduceOnly is not None:
            currOrder.setReduceOnly(reduceOnly)
        if price is not None:
            currOrder.setPrice(price)
        if newClientOrderId is not None:
            currOrder.setNewClientOrderId(newClientOrderId)
        if stopPrice is not None:
            currOrder.setStopPrice(stopPrice)
        if closePosition is not None:
            currOrder.setClosePosition(closePosition)
        if activationPrice is not None:
            currOrder.setActivationPrice(activationPrice)
        if callbackRate is not None:
            currOrder.setCallbackRate(callbackRate)
        if workingType is not None:
            currOrder.setWorkingType(workingType)
        if priceProtect is not None:
            currOrder.setPriceProtect(priceProtect)
        if newOrderRespType is not None:
            currOrder.setNewOrderRespType(newOrderRespType)
        if recvWindow is not None:
            currOrder.setRecvWindow(recvWindow)
        if extraParams is not None:
            currOrder.setExtraParams(extraParams)
        return currOrder

    def createAndTestSpotOrder(self, symbol, side, orderType, quantity=None, price=None, timeInForce=None,
                               stopPrice=None, icebergQty=None, newOrderRespType=None, recvWindow=None,
                               newClientOrderId=None):

        currOrder = self.setSpotOrderData(icebergQty, newClientOrderId, newOrderRespType, orderType, price, quantity,
                                          recvWindow, side, stopPrice, symbol, timeInForce)

        self.exchange.testSpotOrder(currOrder)

        return currOrder

    def makeSpotOrder(self, orderData):
        return self.exchange.makeSpotOrder(orderData)

    def getSymbolOrders(self, symbol, futures=False, orderId=None, startTime=None, endTime=None, limit=None):
        return self.exchange.getSymbolOrders(symbol=symbol, futures=futures, orderId=orderId, startTime=startTime,
                                             endTime=endTime, limit=limit)

    def getOpenOrders(self, symbol, futures=False):
        return self.exchange.getOpenOrders(symbol, futures)

    def getOrder(self, symbol, orderId=None, localOrderId=None, futures=False):
        return self.exchange.getOrder(symbol, orderId, localOrderId, futures=futures)

    def cancelAllSymbolOpenOrders(self, symbol, futures=False):
        return self.exchange.cancelAllSymbolOpenOrders(symbol, futures)

    def cancelOrder(self, symbol, orderId=None, localOrderId=None, futures=False):
        return self.exchange.cancelOrder(symbol, orderId, localOrderId, futures)

    def getTradingFees(self):
        return self.exchange.getTradingFees()

    def getSymbolTickerPrice(self, symbol, futures=False):
        return self.exchange.getSymbolTickerPrice(symbol, futures)

    def getSymbolKlines(self, symbol, interval, startTime=None, endTime=None, limit=None, futures=False, blvtnav=False,
                        convertDateTime=False, doClean=False, toCleanDataframe=False):
        return self.exchange.getSymbolKlines(symbol, interval, startTime, endTime, limit, futures, blvtnav,
                                             convertDateTime, doClean, toCleanDataframe)

    def getExchangeTime(self, futures=False):
        return self.exchange.getExchangeTime(futures)

    def createAndTestFuturesOrder(self, symbol, side, orderType, positionSide=None, timeInForce=None, quantity=None,
                                  reduceOnly=None, price=None, newClientOrderId=None,
                                  stopPrice=None, closePosition=None, activationPrice=None, callbackRate=None,
                                  workingType=None, priceProtect=None, newOrderRespType=None,
                                  recvWindow=None, extraParams=None):
        currOrder = self.setFuturesOrderData(activationPrice, callbackRate, closePosition, extraParams,
                                             newClientOrderId, newOrderRespType, orderType, positionSide, price,
                                             priceProtect, quantity, recvWindow, reduceOnly, side, stopPrice, symbol,
                                             timeInForce, workingType)

        self.exchange.testFuturesOrder(currOrder)

        return currOrder

    def makeFuturesOrder(self, futuresOrderData):
        return self.exchange.makeFuturesOrder(futuresOrderData)

    def makeBatchFuturesOrder(self, batchOrders):
        return self.exchange.makeBatchFuturesOrder(batchOrders)

    def cancelAllSymbolFuturesOrdersWithCountDown(self, symbol, countdownTime):
        return self.exchange.cancellAllSymbolFuturesOrdersWithCountDown(symbol, countdownTime)

    def changeInitialLeverage(self, symbol, leverage):
        return self.exchange.changeInitialLeverage(symbol, leverage)

    def changeMarginType(self, symbol, marginType, params=None):
        return self.exchange.changeMarginType(symbol, marginType, params)

    def changePositionMargin(self, symbol, amount, marginType):
        return self.exchange.changePositionMargin(symbol, amount, marginType)

    def getPosition(self):
        return self.exchange.getPosition()

    def spotBestBidAsks(self, symbol=None):
        return self.exchange.spotBestBidAsks(symbol)

    def getSymbolOrderBook(self, symbol, limit=None, futures=False):
        return self.exchange.getSymbolOrderBook(symbol, limit, futures)

    def getSymbolRecentTrades(self, symbol, limit=None, futures=False):
        return self.exchange.getSymbolRecentTrades(symbol, limit, futures)

    def symbolAccountTradeHistory(self, symbol, futures=False, fromId=None, limit=None):
        return self.exchange.symbolAccountTradeHistory(symbol=symbol, futures=futures, fromId=fromId, limit=limit)

    def makeSlTpLimitFuturesOrder(self, symbol, orderSide, quantity=None, quoteQuantity=None, enterPrice=None,
                                  takeProfit=None, stopLoss=None, leverage=None, marginType=None, cancelDelaySec=None):

        symbolInfo = self.getSymbolMinTrade(symbol=symbol, futures=True)
        # stepQuantity = len(str(symbolInfo['precisionStep'])) - 2

        quantity = self._getQuantity(enterPrice, quantity, quoteQuantity, symbolInfo['precisionStep'])
        self._setLeverage(leverage, symbol)
        self._setMarginType(marginType, symbol)
        cancelIfNotOpened, doPutTpSl, params = self._setTpSlParams(cancelDelaySec, orderSide, stopLoss, takeProfit)

        order = self.testAndMakeFuturesOrder(enterPrice, orderSide, quantity, symbol)

        print('Main order sent')

        watcherProc = multiprocessing.Process(target=watchFuturesLimitTrigger,
                                              args=(self, symbol, order['orderId'], doPutTpSl, cancelIfNotOpened,
                                                    params))
        watcherProc.start()
        # watchFuturesLimitTrigger(self, symbol, order['orderId'], True, False, params)
        return order

    @staticmethod
    def _getQuantity(enterPrice, quantity, quoteQuantity, stepPercision):
        if (quantity is not None and quoteQuantity is not None) or (quantity is None and quoteQuantity is None):
            raise ValueError('Specify either quantity or quoteQuantity and not both')
        if quantity is None:
            if float(stepPercision) > 0.5:
                quantity = round(quoteQuantity / enterPrice, len(str(float(stepPercision))) - 3)
            else:
                quantity = round(quoteQuantity / enterPrice, len(str(float(stepPercision))) - 2)
        return quantity

    def testAndMakeFuturesOrder(self, enterPrice, orderSide, quantity, symbol):
        mainOrder = self.createAndTestFuturesOrder(symbol, orderSide.upper(), 'LIMIT', quantity=str(quantity),
                                                   price=str(enterPrice), timeInForce='GTC')
        order = self.makeFuturesOrder(mainOrder)
        return order

    @staticmethod
    def _setTpSlParams(cancelDelaySec, orderSide, stopLoss, takeProfit):
        cancelIfNotOpened = True if cancelDelaySec is not None else False
        doPutTpSl = True if takeProfit is not None or stopLoss is not None else False
        tpSlOrderSide = 'BUY' if orderSide.upper() == 'SELL' else 'SELL'
        params = {'tpSlOrderSide': tpSlOrderSide, 'takeProfit': takeProfit, 'stopLoss': stopLoss,
                  'cancelDelaySec': cancelDelaySec}
        return cancelIfNotOpened, doPutTpSl, params

    def _setMarginType(self, marginType, symbol):
        try:
            self.exchange.changeMarginType(symbol, marginType)
        except BinanceApiException:
            pass

    def _setLeverage(self, leverage, symbol):
        setLeverageResult = self.changeInitialLeverage(symbol, leverage)
        if not (setLeverageResult['leverage'] == leverage):
            raise ConnectionError('Could not change leverage.')

    def getPositionInfo(self, symbol=None):
        return self.exchange.getPositionInfo(symbol)

    def getSymbolMinTrade(self, symbol, futures=False):
        return self.exchange.getSymbolMinTrade(symbol, futures)



import json
import logging

import pytest

from TradeGates.TradeGate import TradeGate

loglevel = logging.INFO
logging.basicConfig(level=loglevel)
log = logging.getLogger(__name__)


@pytest.fixture
def getGates():
    gates = []
    with open('./config.json') as f:
        config = json.load(f)

    for key in config.keys():
        gates.append(TradeGate(config[key], sandbox=True))

    return gates


@pytest.mark.skip(reason="Only works with main network")
def testTradingFees(getGates):
    for gate in getGates:
        tradingFees = gate.getTradingFees()
        # print('\nTrading fees from {} exchange: {}'.format(gate.exchangeName, tradingFees))

        assert tradingFees is not None, 'Problem in fetching trading fees from {} exchange.'.format(gate.exchangeName)


def testRecentTrades(getGates):
    for gate in getGates:
        symbolRecentPrice = gate.getSymbolRecentTrades('BTCUSDT')
        # print('\nBTCUSDT recent trades from {} exchange: {}'.format(gate.exchangeName, symbolRecentPrice[0]))

        assert symbolRecentPrice is not None, 'Problem in fetching symbol latest price from {} exchange.'.format(
            gate.exchangeName)

        futuresSymbolRecentPrice = gate.getSymbolRecentTrades('BTCUSDT', futures=True)
        # print(
        #     '\nBTCUSDT futures recent trades from {} exchange: {}'.format(gate.exchangeName,
        #                                                                   futuresSymbolRecentPrice[0]))

        assert futuresSymbolRecentPrice is not None, 'Problem in fetching symbol latest price from {} exchange.'.format(
            gate.exchangeName)


def testTickerPrice(getGates):
    for gate in getGates:
        symbolTickerPrice = gate.getSymbolTickerPrice('BTCUSDT')
        # print('\nBTCUSDT ticker Price from {} exchange: {}'.format(gate.exchangeName, symbolTickerPrice))

        assert symbolTickerPrice is not None, 'Problem in fetching symbol ticker price from {} exchange.'.format(
            gate.exchangeName)

        symbolTickerPrice = gate.getSymbolTickerPrice('BTCUSDT', futures=True)
        # print('\nBTCUSDT ticker price from futures market of {} exchange: {}'.format(gate.exchangeName,
        # symbolTickerPrice))

        assert symbolTickerPrice is not None, 'Problem in fetching symbol ticker price from {} exchange.'.format(
            gate.exchangeName)


def testKlines(getGates):
    for gate in getGates:
        # print('\nBTCUSDT candles: {}'.format(gate.getSymbolKlines('BTCUSDT', '1m', limit=10)))
        spotData = gate.getSymbolKlines('BTCUSDT', '15m', limit=10, futures=False, toCleanDataframe=True)
        assert spotData is not None, 'Problem in fetching spot market candle data from {} exchange.'.format(
            gate.exchangeName)
        assert len(spotData) == 10, 'Length of spot market candle data is incorrect from {} exchange.'.format(
            gate.exchangeName)

        futuresData = gate.getSymbolKlines('BTCUSDT', '15m', limit=10, futures=True, toCleanDataframe=True)
        assert futuresData is not None, 'Problem in fetching spot market candle data from {} exchange.'.format(
            gate.exchangeName)
        assert len(futuresData) == 10, 'Length of spot market candle data is incorrect from {} exchange.'.format(
            gate.exchangeName)

        assert futuresData.shape == (10, 7), '7 columns were excpected, but failed from {} exchange.'.format(
            gate.exchangeName)


def testExchangeTime(getGates):
    for gate in getGates:
        exchangeTime = gate.getExchangeTime()
        # print('\nExchange time from spot market of {} exchange: {}'.format(gate.exchangeName, exchangeTime))
        assert exchangeTime is not None, 'Problem in fetching exchange time from {} exchange.'.format(gate.exchangeName)

        exchangeTime = gate.getExchangeTime(futures=True)
        # print('\nExchange time from futures market of {} exchange: {}'.format(gate.exchangeName, exchangeTime))
        assert exchangeTime is not None, 'Problem in fetching exchange time from {} exchange.'.format(gate.exchangeName)



from binance_f.constant.system import RestApiDefine
from binance_f.impl.restapiinvoker import call_sync
from binance_f.impl.restapirequestimpl import RestApiRequestImpl
from binance_f.model.constant import *


class RequestClient(object):

    def __init__(self, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Binance.
            secret_key: The private key applied from Binance.
            server_url: The URL name like "https://api.binance.com".
        """
        api_key = None
        secret_key = None
        url = RestApiDefine.Url
        if "api_key" in kwargs:
            api_key = kwargs["api_key"]
        if "secret_key" in kwargs:
            secret_key = kwargs["secret_key"]
        if "url" in kwargs:
            url = kwargs["url"]
        try:
            self.request_impl = RestApiRequestImpl(api_key, secret_key, url)
        except Exception:
            pass
        self.limits = {}

    def refresh_limits(self, limits):
        for k, v in limits.items():
            self.limits[k] = v

    def get_servertime(self) -> any:
        """
        Check Server Time

        GET /fapi/v1/time

        Test connectivity to the Rest API and get the current server time.
        """
        response = call_sync(self.request_impl.get_servertime())
        self.refresh_limits(response[1])
        return response[0]

    def get_exchange_information(self) -> any:
        """
        Exchange Information (MARKET_DATA)

        GET /fapi/v1/exchangeInfo

        Current exchange trading rules and symbol information
        """
        response = call_sync(self.request_impl.get_exchange_information())
        self.refresh_limits(response[1])
        return response[0]

    def get_order_book(self, symbol: 'str', limit: 'int' = None) -> any:
        """
        Order Book (MARKET_DATA)

        GET /fapi/v1/depth

        Adjusted based on the limit:
        """
        response = call_sync(self.request_impl.get_order_book(symbol, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_recent_trades_list(self, symbol: 'str', limit: 'int' = None) -> any:
        """
        Recent Trades List (MARKET_DATA)

        GET /fapi/v1/trades

        Get recent trades (up to last 500).
        """
        response = call_sync(self.request_impl.get_recent_trades_list(symbol, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_old_trade_lookup(self, symbol: 'str', limit: 'int' = None, fromId: 'long' = None) -> any:
        """
        Old Trades Lookup (MARKET_DATA)

        GET /fapi/v1/historicalTrades

        Get older market historical trades.
        """
        response = call_sync(self.request_impl.get_old_trade_lookup(symbol, limit, fromId))
        self.refresh_limits(response[1])
        return response[0]

    def get_aggregate_trades_list(self, symbol: 'str', fromId: 'long' = None,
                                  startTime: 'long' = None, endTime: 'long' = None, limit: 'int' = None) -> any:
        """
        Compressed/Aggregate Trades List (MARKET_DATA)

        GET /fapi/v1/aggTrades

        Get compressed, aggregate trades. Trades that fill at the time, from the same order, 
        with the same price will have the quantity aggregated.
        """
        response = call_sync(self.request_impl.get_aggregate_trades_list(symbol, fromId, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_candlestick_data(self, symbol: 'str', interval: 'CandlestickInterval',
                             startTime: 'long' = None, endTime: 'long' = None, limit: 'int' = None) -> any:
        """
        Kline/Candlestick Data (MARKET_DATA)

        GET /fapi/v1/klines

        Kline/candlestick bars for a symbol. Klines are uniquely identified by their open time.
        """
        response = call_sync(self.request_impl.get_candlestick_data(symbol, interval, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_mark_price(self, symbol: 'str' = None) -> any:
        """
        Mark Price (MARKET_DATA)

        GET /fapi/v1/premiumIndex

        Mark Price and Funding Rate
        """
        response = call_sync(self.request_impl.get_mark_price(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def get_funding_rate(self, symbol: 'str', startTime: 'long' = None, endTime: 'str' = None,
                         limit: 'int' = None) -> any:
        """
        Get Funding Rate History (MARKET_DATA)

        GET /fapi/v1/fundingRate
        """
        response = call_sync(self.request_impl.get_funding_rate(symbol, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_ticker_price_change_statistics(self, symbol: 'str' = None) -> any:
        """
        24hr Ticker Price Change Statistics (MARKET_DATA)

        GET /fapi/v1/ticker/24hr

        24 hour rolling window price change statistics.
        Careful when accessing this with no symbol.
        """
        response = call_sync(self.request_impl.get_ticker_price_change_statistics(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def get_symbol_price_ticker(self, symbol: 'str' = None) -> any:
        """
        Symbol Price Ticker (MARKET_DATA)

        GET /fapi/v1/ticker/price

        Latest price for a symbol or symbols.
        """
        response = call_sync(self.request_impl.get_symbol_price_ticker(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def get_symbol_orderbook_ticker(self, symbol: 'str' = None) -> any:
        """
        Symbol Order Book Ticker (MARKET_DATA)

        GET /fapi/v1/ticker/bookTicker

        Best price/qty on the order book for a symbol or symbols.
        """
        response = call_sync(self.request_impl.get_symbol_orderbook_ticker(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def get_liquidation_orders(self, symbol: 'str' = None, startTime: 'long' = None, endTime: 'str' = None,
                               limit: 'int' = None) -> any:
        """
        Get all Liquidation Orders (MARKET_DATA)

        GET /fapi/v1/allForceOrders
        """
        response = call_sync(self.request_impl.get_liquidation_orders(symbol, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_open_interest(self, symbol: 'str') -> any:
        """
        Symbol Open Interest (MARKET_DATA)

        GET /fapi/v1/openInterest

        Get present open interest of a specific symbol.
        """
        response = call_sync(self.request_impl.get_open_interest(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def change_position_mode(self, dualSidePosition: 'boolean' = None) -> any:
        """
        Change Current Position Mode (TRADE)

        POST /fapi/v1/positionSide/dual (HMAC SHA256)

        Change user's position mode (Hedge Mode or One-way Mode ) on EVERY symbol
        """
        response = call_sync(self.request_impl.change_position_mode(dualSidePosition))
        self.refresh_limits(response[1])
        return response[0]

    def get_position_mode(self) -> any:
        """
        Get Current Position Mode (USER_DATA)

        GET /fapi/v1/positionSide/dual (HMAC SHA256)

        Get user's position mode (Hedge Mode or One-way Mode ) on EVERY symbol
        """
        response = call_sync(self.request_impl.get_position_mode())
        self.refresh_limits(response[1])
        return response[0]

    def post_order(self, symbol: 'str', side: 'OrderSide', ordertype: 'OrderType',
                   timeInForce: 'TimeInForce' = TimeInForce.INVALID, quantity: 'float' = None,
                   reduceOnly: 'boolean' = None, price: 'float' = None,
                   newClientOrderId: 'str' = None, stopPrice: 'float' = None,
                   workingType: 'WorkingType' = WorkingType.INVALID, closePosition: 'boolean' = None,
                   positionSide: 'PositionSide' = PositionSide.INVALID, callbackRate: 'float' = None,
                   activationPrice: 'float' = None, priceProtect: 'boolean' = None,
                   newOrderRespType: 'OrderRespType' = OrderRespType.INVALID) -> any:
        """
        New Order (TRADE)

        POST /fapi/v1/order (HMAC SHA256)

        Send in a new order.
        """
        response = call_sync(
            self.request_impl.post_order(symbol, side, ordertype, timeInForce, quantity, reduceOnly, price,
                                         newClientOrderId, stopPrice,
                                         workingType, closePosition, positionSide, callbackRate, activationPrice,
                                         priceProtect, newOrderRespType))
        self.refresh_limits(response[1])
        return response[0]

    def post_batch_order(self, batchOrders: 'list') -> any:
        """
        New Orders Batch (TRADE)

        POST /fapi/v1/batchOrder (HMAC SHA256)

        Send in multiple new orders.
        """
        response = call_sync(
            self.request_impl.post_batch_order(batchOrders))
        self.refresh_limits(response[1])
        return response[0]

    def get_order(self, symbol: 'str', orderId: 'long' = None, origClientOrderId: 'str' = None) -> any:
        """
        Query Order (USER_DATA)

        GET /fapi/v1/order (HMAC SHA256)

        Check an order's status.
        """
        response = call_sync(self.request_impl.get_order(symbol, orderId, origClientOrderId))
        self.refresh_limits(response[1])
        return response[0]

    def cancel_order(self, symbol: 'str', orderId: 'long' = None, origClientOrderId: 'str' = None) -> any:
        """
        Cancel Order (TRADE)

        DELETE /fapi/v1/order (HMAC SHA256)

        Cancel an active order.
        """
        response = call_sync(self.request_impl.cancel_order(symbol, orderId, origClientOrderId))
        self.refresh_limits(response[1])
        return response[0]

    def cancel_all_orders(self, symbol: 'str') -> any:
        """
        Cancel All Open Orders (TRADE)

        DELETE /fapi/v1/allOpenOrders (HMAC SHA256)
        """
        response = call_sync(self.request_impl.cancel_all_orders(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def cancel_list_orders(self, symbol: 'str', orderIdList: 'list' = None,
                           origClientOrderIdList: 'list' = None) -> any:
        """
        Cancel Multiple Orders (TRADE)

        DELETE /fapi/v1/batchOrders (HMAC SHA256)
        """
        response = call_sync(self.request_impl.cancel_list_orders(symbol, orderIdList, origClientOrderIdList))
        self.refresh_limits(response[1])
        return response[0]

    def get_open_orders(self, symbol: 'str' = None) -> any:
        """
        Current Open Orders (USER_DATA)

        GET /fapi/v1/openOrders (HMAC SHA256)

        Get all open orders on a symbol. Careful when accessing this with no symbol.
        """
        response = call_sync(self.request_impl.get_open_orders(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def get_all_orders(self, symbol: 'str', orderId: 'long' = None, startTime: 'long' = None,
                       endTime: 'long' = None, limit: 'int' = None) -> any:
        """
        All Orders (USER_DATA)

        GET /fapi/v1/allOrders (HMAC SHA256)

        Get all account orders; active, canceled, or filled.
        """
        response = call_sync(self.request_impl.get_all_orders(symbol, orderId, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_balance(self) -> any:
        """
        Future Account Balance (USER_DATA)

        Get /fapi/v1/balance (HMAC SHA256)
        """
        response = call_sync(self.request_impl.get_balance())
        self.refresh_limits(response[1])
        return response[0]

    def get_account_information(self) -> any:
        """
        Account Information (USER_DATA)

        GET /fapi/v1/account (HMAC SHA256)

        Get current account information.
        """
        response = call_sync(self.request_impl.get_account_information())
        self.refresh_limits(response[1])
        return response[0]

    def change_initial_leverage(self, symbol: 'str', leverage: 'int') -> any:
        """
        Change Initial Leverage (TRADE)

        POST /fapi/v1/leverage (HMAC SHA256)

        Change user's initial leverage of specific symbol market.
        """
        response = call_sync(self.request_impl.change_initial_leverage(symbol, leverage))
        self.refresh_limits(response[1])
        return response[0]

    def change_margin_type(self, symbol: 'str', marginType: 'FuturesMarginType') -> any:
        """
        Change Margin Type (TRADE)

        POST /fapi/v1/marginType (HMAC SHA256)
        """
        response = call_sync(self.request_impl.change_margin_type(symbol, marginType))
        self.refresh_limits(response[1])
        return response[0]

    def change_position_margin(self, symbol: 'str', amount: 'float', type: 'int') -> any:
        """
        Modify Isolated Position Margin (TRADE)

        POST /fapi/v1/positionMargin (HMAC SHA256)
        """
        response = call_sync(self.request_impl.change_position_margin(symbol, amount, type))
        self.refresh_limits(response[1])
        return response[0]

    def get_position_margin_change_history(self, symbol: 'str', type: 'int' = None, startTime: 'int' = None,
                                           endTime: 'int' = None, limit: 'int' = None) -> any:
        """
        Get Position Margin Change History (TRADE)

        GET /fapi/v1/positionMargin/history (HMAC SHA256)
        """
        response = call_sync(
            self.request_impl.get_position_margin_change_history(symbol, type, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_position(self) -> any:
        """
        Position Information (USER_DATA)

        GET /fapi/v1/positionRisk (HMAC SHA256) Get current account information.
        """
        response = call_sync(self.request_impl.get_position())
        self.refresh_limits(response[1])
        return response[0]

    def get_account_trades(self, symbol: 'str', startTime: 'long' = None, endTime: 'long' = None,
                           fromId: 'long' = None, limit: 'int' = None) -> any:
        """
        Account Trade List (USER_DATA)

        GET /fapi/v1/userTrades (HMAC SHA256)

        Get trades for a specific account and symbol.
        """
        response = call_sync(self.request_impl.get_account_trades(symbol, startTime, endTime, fromId, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_income_history(self, symbol: 'str' = None, incomeType: 'IncomeType' = IncomeType.INVALID,
                           startTime: 'long' = None, endTime: 'long' = None, limit: 'int' = None) -> any:
        """
        Get Income History(USER_DATA)

        GET /fapi/v1/income (HMAC SHA256)
        """
        response = call_sync(self.request_impl.get_income_history(symbol, incomeType, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def start_user_data_stream(self) -> any:
        """
        Start User Data Stream (USER_STREAM)

        POST /fapi/v1/listenKey (HMAC SHA256)

        Start a new user data stream. The stream will close after 60 minutes unless a keepalive is sent. 
        If the account has an active listenKey, 
        that listenKey will be returned and its validity will be extended for 60 minutes.
        """
        response = call_sync(self.request_impl.start_user_data_stream())
        self.refresh_limits(response[1])
        return response[0]

    def keep_user_data_stream(self) -> any:
        """
        Keepalive User Data Stream (USER_STREAM)

        PUT /fapi/v1/listenKey (HMAC SHA256)

        Keepalive a user data stream to prevent a time out. User data streams will close after 60 minutes. 
        It's recommended to send a ping about every 60 minutes.
        """
        response = call_sync(self.request_impl.keep_user_data_stream())
        self.refresh_limits(response[1])
        return response[0]

    def close_user_data_stream(self) -> any:
        """
        Close User Data Stream (USER_STREAM)

        DELETE /fapi/v1/listenKey (HMAC SHA256)

        Close out a user data stream.
        """
        response = call_sync(self.request_impl.close_user_data_stream())
        self.refresh_limits(response[1])
        return response[0]

    def get_open_interest_stats(self, symbol: 'str', period: 'str', startTime: 'str' = None, endTime: 'str' = None,
                                limit: 'int' = 30) -> any:
        """
        Open Interest Statistics (MARKET_DATA)

        GET /futures/data/openInterestHist

        """
        response = call_sync(self.request_impl.get_open_interest_stats(symbol, period, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_top_long_short_accounts(self, symbol: 'str', period: 'str', startTime: 'str' = None, endTime: 'str' = None,
                                    limit: 'int' = 30) -> any:
        """
        Top Trader Long/Short Ratio (Accounts) (MARKET_DATA)

        GET /futures/data/topLongShortAccountRatio

        """
        response = call_sync(self.request_impl.get_top_long_short_accounts(symbol, period, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_top_long_short_positions(self, symbol: 'str', period: 'str', startTime: 'str' = None, endTime: 'str' = None,
                                     limit: 'int' = 30) -> any:
        """
        Top Trader Long/Short Ratio (Positions)

        GET /futures/data/topLongShortPositionRatio

        """
        response = call_sync(self.request_impl.get_top_long_short_positions(symbol, period, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_global_long_short_accounts(self, symbol: 'str', period: 'str', startTime: 'str' = None,
                                       endTime: 'str' = None, limit: 'int' = 30) -> any:
        """
        Long/Short Ratio (MARKET_DATA)

        GET /futures/data/globalLongShortAccountRatio

        """
        response = call_sync(
            self.request_impl.get_global_long_short_accounts(symbol, period, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_taker_buy_sell_ratio(self, symbol: 'str', period: 'str', startTime: 'str' = None, endTime: 'str' = None,
                                 limit: 'int' = 30) -> any:
        """
        Taker Buy/Sell Volume(MARKET_DATA)

        GET /futures/data/takerlongshortRatio

        """
        response = call_sync(self.request_impl.get_taker_buy_sell_ratio(symbol, period, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_blvt_nav_candlestick_data(self, symbol: 'str', interval: 'CandlestickInterval',
                                      startTime: 'long' = None, endTime: 'long' = None, limit: 'int' = None) -> any:
        """
        Historical BLVT NAV Kline/Candlestick (MARKET_DATA)

        GET /fapi/v1/lvtKlines

        The BLVT NAV system is based on Binance Futures, so the endpoint is based on fapi
        """
        response = call_sync(
            self.request_impl.get_blvt_nav_candlestick_data(symbol, interval, startTime, endTime, limit))
        self.refresh_limits(response[1])
        return response[0]

    def get_composite_index_info(self, symbol: 'str') -> any:
        """
        Composite Index Symbol Information (MARKET_DATA)

        GET /fapi/v1/indexInfo
        """
        response = call_sync(self.request_impl.get_composite_index_info(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def auto_cancel_all_orders(self, symbol: 'str', countdownTime: 'long') -> any:
        """
        Auto-Cancel All Open Orders (TRADE)

        POST /fapi/v1/countdownCancelAll (HMAC SHA256)
        """
        response = call_sync(self.request_impl.auto_cancel_all_orders(symbol, countdownTime))
        self.refresh_limits(response[1])
        return response[0]

    def get_balance_v2(self) -> any:
        """
        Future Account Balance (USER_DATA)

        Get /fapi/v2/balance (HMAC SHA256)
        """
        response = call_sync(self.request_impl.get_balance_v2())
        self.refresh_limits(response[1])
        return response[0]

    def get_account_information_v2(self) -> any:
        """
        Account Information (USER_DATA)

        GET /fapi/v2/account (HMAC SHA256)

        Get current account information.
        """
        response = call_sync(self.request_impl.get_account_information_v2())
        self.refresh_limits(response[1])
        return response[0]

    def get_position_v2(self, symbol: 'str' = None) -> any:
        """
        Position Information (USER_DATA)

        GET /fapi/v2/positionRisk (HMAC SHA256) Get current account information.
        """
        response = call_sync(self.request_impl.get_position_v2(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def get_leverage_bracket(self, symbol: 'str' = None) -> any:
        """
        Notional and Leverage Brackets (USER_DATA)

        GET /fapi/v1/leverageBracket
        """
        response = call_sync(self.request_impl.get_leverage_bracket(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def get_adl_quantile(self, symbol: 'str' = None) -> any:
        """
        Position ADL Quantile Estimation (USER_DATA)

        GET /fapi/v1/adlQuantile
        """
        response = call_sync(self.request_impl.get_adl_quantile(symbol))
        self.refresh_limits(response[1])
        return response[0]

    def get_api_trading_stats(self, symbol: 'str' = None) -> any:
        """
        User API Trading Quantitative Rules Indicators (USER_DATA)

        GET /fapi/v1/apiTradingStatus
        """
        response = call_sync(self.request_impl.get_api_trading_stats(symbol))
        self.refresh_limits(response[1])
        return response[0]



import sys


import time




class PrintDate:
    @staticmethod
    def timestamp_to_date(ts_minsecond):
        try:
            ts_minsecond = int(ts_minsecond)
            time_local = time.localtime(int(ts_minsecond / 1000))
            dt = time.strftime("%Y-%m-%d %H:%M:%S", time_local)
            print("ping " + str(ts_minsecond) + ":" + dt)
        except Exception as e:
            print(e)

if __name__ == "__main__":
    ping_ts = 1569319465421
    PrintDate.timestamp_to_date(ping_ts)
    PrintDate.timestamp_to_date(int(ping_ts), ("ping " + str(ping_ts)))




import urllib.parse

from binance_f.constant.system import WebSocketDefine
from binance_f.impl.websocketrequestimpl import WebsocketRequestImpl
from binance_f.impl.websocketconnection import WebsocketConnection
from binance_f.impl.websocketwatchdog import WebSocketWatchDog
from binance_f.impl.restapirequestimpl import RestApiRequestImpl
from binance_f.model import *
from binance_f.model.constant import *

# For develop
from binance_f.base.printobject import *

class SubscriptionClient(object):

    def __init__(self, **kwargs):
        """
        Create the subscription client to subscribe the update from server.

        :param kwargs: The option of subscription connection.
            api_key: The public key applied from Binance.
            secret_key: The private key applied from Binance.
            uri: Set the URI for subscription.
            is_auto_connect: When the connection lost is happening on the subscription line, specify whether the client
                            reconnect to server automatically. The connection lost means:
                                Caused by network problem
                                The connection close triggered by server (happened every 24 hours)
                            No any message can be received from server within a specified time, see receive_limit_ms
            receive_limit_ms: Set the receive limit in millisecond. If no message is received within this limit time,
                            the connection will be disconnected.
            connection_delay_failure: If auto reconnect is enabled, specify the delay time before reconnect.
        """
        api_key = None
        secret_key = None
        if "api_key" in kwargs:
            api_key = kwargs["api_key"]
        if "secret_key" in kwargs:
            secret_key = kwargs["secret_key"]
        self.__api_key = api_key
        self.__secret_key = secret_key
        self.websocket_request_impl = WebsocketRequestImpl(self.__api_key)
        self.connections = list()
        self.uri = WebSocketDefine.Uri
        is_auto_connect = True
        receive_limit_ms = 60000
        connection_delay_failure = 1
        if "uri" in kwargs:
            self.uri = kwargs["uri"]
        if "is_auto_connect" in kwargs:
            is_auto_connect = kwargs["is_auto_connect"]
        if "receive_limit_ms" in kwargs:
            receive_limit_ms = kwargs["receive_limit_ms"]
        if "connection_delay_failure" in kwargs:
            connection_delay_failure = kwargs["connection_delay_failure"]
        self.__watch_dog = WebSocketWatchDog(is_auto_connect, receive_limit_ms, connection_delay_failure)

    def __create_connection(self, request):
        connection = WebsocketConnection(self.__api_key, self.__secret_key, self.uri, self.__watch_dog, request)
        self.connections.append(connection)
        connection.connect()

    def unsubscribe_all(self):
        for conn in self.connections:
            conn.close()
        self.connections.clear()

    def subscribe_aggregate_trade_event(self, symbol: 'str', callback, error_handler=None):
        """
        Aggregate Trade Streams

        The Aggregate Trade Streams push trade information that is aggregated for a single taker order every 100 milliseconds.

        Stream Name: <symbol>@aggTrade
        """
        request = self.websocket_request_impl.subscribe_aggregate_trade_event(symbol, callback, error_handler)
        self.__create_connection(request)

    def subscribe_mark_price_event(self, symbol: 'str', callback, error_handler=None):
        """
        Mark Price Stream

        Mark price for a single symbol pushed every 3 secends.

        Stream Name: <symbol>@markPrice
        """
        request = self.websocket_request_impl.subscribe_mark_price_event(symbol, callback, error_handler)
        self.__create_connection(request)

    def subscribe_continuous_candlestick_event(self, pair: 'str', contract_type: 'ContractType', interval: 'CandlestickInterval',
                                               callback,
                                               error_handler=None):
        """
        Continuous Kline/Candlestick Streams

        The continuous Kline/Candlestick Stream push updates to the current klines/candlestick every 250 milliseconds (if existing).

        Stream Name: <pair><contractType>@continuousKline_<interval>
        """
        request = self.websocket_request_impl.subscribe_continuous_candlestick_event(pair, contract_type, interval,
                                                                                     callback, error_handler)
        self.__create_connection(request)
    
    def subscribe_candlestick_event(self, symbol: 'str', interval: 'CandlestickInterval', callback,
                                    error_handler=None):
        """
        Kline/Candlestick Streams

        The Kline/Candlestick Stream push updates to the current klines/candlestick every 250 milliseconds (if existing).

        Stream Name: <symbol>@kline_<interval>
        """
        request = self.websocket_request_impl.subscribe_candlestick_event(symbol, interval, callback, error_handler)
        self.__create_connection(request)
           
    def subscribe_symbol_miniticker_event(self, symbol: 'str', callback, error_handler=None):
        """
        Individual Symbol Mini Ticker Stream

        24hr rolling window mini-ticker statistics for a single symbol pushed every 3 seconds. These are NOT the statistics of the UTC day, 
        but a 24hr rolling window from requestTime to 24hrs before.

        Stream Name: <symbol>@miniTicker
        """
        request = self.websocket_request_impl.subscribe_symbol_miniticker_event(symbol, callback, error_handler)
        self.__create_connection(request)
  
    def subscribe_all_miniticker_event(self, callback, error_handler=None):
        """
        All Market Mini Tickers Stream

        24hr rolling window mini-ticker statistics for all symbols pushed every 3 seconds. 
        These are NOT the statistics of the UTC day, but a 24hr rolling window from requestTime to 24hrs before. 
        Note that only tickers that have changed will be present in the array.

        Stream Name: !miniTicker@arr
        """
        request = self.websocket_request_impl.subscribe_all_miniticker_event(callback, error_handler)
        self.__create_connection(request)

    def subscribe_symbol_ticker_event(self, symbol: 'str', callback, error_handler=None):
        """
        Individual Symbol Ticker Streams

        24hr rollwing window ticker statistics for a single symbol pushed every 3 seconds. These are NOT the statistics of the UTC day, 
        but a 24hr rolling window from requestTime to 24hrs before.

        Stream Name: <symbol>@ticker
        """
        request = self.websocket_request_impl.subscribe_symbol_ticker_event(symbol, callback, error_handler)
        self.__create_connection(request)
      
    def subscribe_all_ticker_event(self, callback, error_handler=None):
        """
        All Market Tickers Stream

        24hr rollwing window ticker statistics for all symbols pushed every 3 seconds. These are NOT the statistics of the UTC day, but a 24hr rolling window from requestTime to 24hrs before. 
        Note that only tickers that have changed will be present in the array.

        Stream Name: !ticker@arr
        """
        request = self.websocket_request_impl.subscribe_all_ticker_event(callback, error_handler)
        self.__create_connection(request)

    def subscribe_symbol_bookticker_event(self, symbol: 'str', callback, error_handler=None):
        """
        Individual Symbol Book Ticker Streams

        Pushes any update to the best bid or ask's price or quantity in real-time for a specified symbol.

        Stream Name: <symbol>@bookTicker
        """
        request = self.websocket_request_impl.subscribe_symbol_bookticker_event(symbol, callback, error_handler)
        self.__create_connection(request)
    
    def subscribe_all_bookticker_event(self, callback, error_handler=None):
        """
        All Book Tickers Stream

        Pushes any update to the best bid or ask's price or quantity in real-time for all symbols.

        Stream Name: !bookTicker
        """
        request = self.websocket_request_impl.subscribe_all_bookticker_event(callback, error_handler)
        self.__create_connection(request)

    def subscribe_symbol_liquidation_event(self, symbol: 'str', callback, error_handler=None):
        """
        Liquidation Order Streams

        The Liquidation Order Streams push force liquidation order information for specific symbol

        Stream Name:  <symbol>@forceOrder
        """
        request = self.websocket_request_impl.subscribe_symbol_liquidation_event(symbol, callback, error_handler)
        self.__create_connection(request)

    def subscribe_all_liquidation_event(self, callback, error_handler=None):
        """
        All Market Liquidation Order Streams

        The All Liquidation Order Streams push force liquidation order information for all symbols in the market.

        Stream Name: !forceOrder@arr
        """
        request = self.websocket_request_impl.subscribe_all_liquidation_event(callback, error_handler)
        self.__create_connection(request)
             
    def subscribe_book_depth_event(self, symbol: 'str', limit: 'int', callback, error_handler=None, update_time: 'UpdateTime' = UpdateTime.INVALID):
        """
        Partial Book Depth Streams

        Top bids and asks, Valid are 5, 10, or 20.

        Stream Names: <symbol>@depth<levels> OR <symbol>@depth<levels>@100ms.
        """
        print(update_time)
        request = self.websocket_request_impl.subscribe_book_depth_event(symbol, limit, update_time, callback, error_handler)
        self.__create_connection(request)

    def subscribe_diff_depth_event(self, symbol: 'str', callback, error_handler=None, update_time: 'UpdateTime' = UpdateTime.INVALID):
        """
        Diff. Depth Stream

        Bids and asks, pushed every 250 milliseconds or 100 milliseconds(if existing)

        Stream Name: <symbol>@depth OR <symbol>@depth@100ms
        """
        request = self.websocket_request_impl.subscribe_diff_depth_event(symbol, update_time, callback, error_handler)
        self.__create_connection(request)

    def subscribe_user_data_event(self, listenKey: 'str', callback, error_handler=None):
        """
        User Data Streams
        """
        request = self.websocket_request_impl.subscribe_user_data_event(listenKey, callback, error_handler)
        self.__create_connection(request)

    def subscribe_all_mark_price_event(self, callback, error_handler=None):
        """
        Mark Price Stream for All market

        Mark price and funding rate for all symbols pushed every 3 seconds or every second.

        Stream Name: !markPrice@arr or !markPrice@arr@1s

        Update Speed: 3000ms or 1000ms
        """
        request = self.websocket_request_impl.subscribe_all_mark_price_event(callback, error_handler)
        self.__create_connection(request)

    def subscribe_blvt_info_event(self, symbol, callback, error_handler=None):
        """
        BLVT Info Streams

        Stream Name: <tokenName>@tokenNav

        Note: tokenName must be uppercase, e.g. "TRXDOWN"

        Update Speed: 300ms
        """
        request = self.websocket_request_impl.subscribe_blvt_info_event(symbol, callback, error_handler)
        self.__create_connection(request)

    def subscribe_blvt_nav_candlestick_event(self, symbol, interval, callback, error_handler=None):
        """
        BLVT NAV Kline/Candlestick Streams

        Stream Name: <tokenName>@nav_Kline_<interval>

        Note: tokenName must be uppercase, e.g. "TRXDOWN"

        Update Speed: 300ms
        """
        request = self.websocket_request_impl.subscribe_blvt_nav_candlestick_event(symbol, interval, callback, error_handler)
        self.__create_connection(request)

    def subscribe_composite_index_event(self, symbol, callback, error_handler=None):
        """
        Composite Index Symbol Information Streams

        Composite index information for index symbols pushed every second.

        Stream Name: <symbol>@compositeIndex

        Update Speed: 1000ms
        """
        request = self.websocket_request_impl.subscribe_composite_index_event(symbol, callback, error_handler)
        self.__create_connection(request)



class Bracket:

    def __init__(self):
        self.bracket = 0
        self.initialLeverage = 0
        self.notionalCap = 0.0
        self.notionalFloor = 0.0
        self.maintMarginRatio = 0.0
        self.cum = 0.0
        
    @staticmethod
    def json_parse(json_data):
        result = Bracket()
        result.bracket = json_data.get_int("bracket")
        result.initialLeverage = json_data.get_int("initialLeverage")
        result.notionalCap = json_data.get_float("notionalCap")
        result.notionalFloor = json_data.get_float("notionalFloor")
        result.maintMarginRatio = json_data.get_float("maintMarginRatio")
        result.cum = json_data.get_float("cum")
        return result

class LeverageBracket:

    def __init__(self):
        self.symbol = ""
        self.brackets = list()
    
    @staticmethod
    def json_parse(json_data):
        result = LeverageBracket()
        result.symbol = json_data.get_string("symbol")

        element_list = list()
        data_list = json_data.get_array("brackets")
        for item in data_list.get_items():
            element = Bracket.json_parse(item)
            element_list.append(element)
        result.brackets = element_list

        return result



import json
import logging

import pytest

from TradeGates.TradeGate import TradeGate

loglevel = logging.INFO
logging.basicConfig(level=loglevel)
log = logging.getLogger(__name__)


@pytest.fixture
def getGates():
    gates = []
    with open('./config.json') as f:
        config = json.load(f)

    for key in config.keys():
        gates.append(TradeGate(config[key], sandbox=True))

    return gates


def testFullBalance(getGates):
    for gate in getGates:
        balance = gate.getBalance()
        # print('\nFull Balance from {} exchange: {}'.format(gate.exchangeName, balance))
        assert balance is not None, 'Problem in fetching balance from {} exchange.'.format(gate.exchangeName)

        errorMessage = 'Bad fetch balance interface for {} exchange,'.format(gate.exchangeName)
        try:
            if not gate.exchangeName == 'Binance':
                if not sorted(list(balance[0].keys())) == sorted(['asset', 'free', 'locked', 'exchangeSpecific']):
                    assert False, errorMessage
            else:
                if not sorted(list(balance[0].keys())) == sorted(['asset', 'free', 'locked']):
                    assert False, errorMessage
        except Exception:
            assert False, errorMessage


def testSingleCoinBalance(getGates):
    for gate in getGates:
        balance = gate.getBalance('BTC')
        # print('\nSingle coin balance from {} exchange: {}'.format(gate.exchangeName, balance))
        assert balance is not None, 'Problem in fetching single coin balance from {} exchange.'.format(
            gate.exchangeName)

        errorMessage = 'Bad fetch single coin balance interface for {} exchange,'.format(gate.exchangeName)
        try:
            if not gate.exchangeName == 'Binance':
                if not sorted(list(balance.keys())) == sorted(['asset', 'free', 'locked', 'exchangeSpecific']):
                    assert False, errorMessage
            else:
                if not sorted(list(balance.keys())) == sorted(['asset', 'free', 'locked']):
                    assert False, errorMessage
        except Exception:
            assert False, errorMessage


def testTradeHistory(getGates):
    for gate in getGates:
        tradeHistory = gate.symbolAccountTradeHistory('BTCUSDT', futures=True)
        # print('\nTrade history from {} exchange: {}'.format(gate.exchangeName, tradeHistory[0]))

        assert tradeHistory is not None, 'Problem in fetching trade history from {} exchange.'.format(gate.exchangeName)

        interface = ['symbol', 'id', 'orderId', 'orderListId', 'price', 'qty', 'quoteQty', 'commission',
                     'commissionAsset', 'time',
                     'isBuyer', 'isMaker', 'isBestMatch']

        errorMessage = 'Bad fetch trade history interface for {} exchange,'.format(gate.exchangeName)
        try:
            if not gate.exchangeName == 'Binance':
                interface.append('exchangeSpecific')
                if not sorted(list(tradeHistory[0].keys())) == sorted(interface):
                    assert False, errorMessage
            else:
                if not sorted(list(tradeHistory[0].keys())) == sorted(interface):
                    assert False, errorMessage
        except Exception:
            assert False, errorMessage



from kucoin_futures.base_request.base_request import KucoinFuturesBaseRestApi


class UserData(KucoinFuturesBaseRestApi):

    def transfer_kucoin_account(self, amount, bizNo=''):
        """
        https://docs.kumex.com/#transfer-funds-to-kucoin-main-account
        :param bizNo:  (Mandatory) A unique ID generated by the user, to ensure the operation is processed by the system only once.
            You are suggested to use UUID
        :type: str
        :param amount: (Mandatory) Amount to be transfered out
        :type:float
        :return:
        {
            "applyId": "5bffb63303aa675e8bbe18f9" //Transfer-out request ID
        }
        """
        params = {'amount': amount}
        if not bizNo:
            bizNo = self.return_unique_id[0:23]
        params['bizNo'] = bizNo
        return self._request('POST', '/api/v1/transfer-out', params=params)

    def transfer_kucoin_account_v2(self, amount, bizNo=''):
        """
        https://docs.kumex.com/#transfer-funds-to-kucoin-main-account-2
        :param bizNo:  (Mandatory) A unique ID generated by the user, to ensure the operation is processed by the system only once.
            You are suggested to use UUID
        :type: str
        :param amount: (Mandatory) Amount to be transfered out
        :type:float
        :return:
        {
            "applyId": "5bffb63303aa675e8bbe18f9" //Transfer-out request ID
        }
        """
        params = {'amount': amount}
        if not bizNo:
            bizNo = self.return_unique_id[0:23]
        params['bizNo'] = bizNo
        return self._request('POST', '/api/v2/transfer-out', params=params)

    def get_Transfer_history(self, **kwargs):
        """
        https://docs.kumex.com/#get-transfer-out-request-records-2
        :param kwargs:  [optional]  status,  startAt, endAt, currentPage , pageSize  and so on
        :return: {'totalNum': 0, 'totalPage': 0, 'pageSize': 50, 'currentPage': 1, 'items': [{
        "applyId": "5cd53be30c19fc3754b60928", //Transfer-out request ID
        "currency": "XBT", //Currency
        "status": "SUCCESS", //Status  PROCESSING, SUCCESS, FAILURE
        "amount": "0.01", //Transaction amount
        "reason": "", //Reason caused the failure
        "offset": 31986850860000, //Offset
        "createdAt": 1557769977000 //Request application time}, ....]}
        """
        params = {}
        if kwargs:
            params.update(kwargs)
        return self._request('GET', '/api/v1/transfer-list', params=params)

    def cancel_Transfer_out(self, applyId):
        """
        https://docs.kumex.com/#cancel-transfer-out-request
        :param applyId: (Mandatory) Transfer ID (Initiate to cancel the transfer-out request)
        :return: {'code': '200000',"data": {
       "applyId": "5bffb63303aa675e8bbe18f9" //Transfer-out request ID
        }  }
        """
        return self._request('DELETE', '/api/v1/cancel/transfer-out?applyId={}'.format(applyId))

    def get_withdrawal_quota(self, currency):
        """
        https://docs.kumex.com/#get-withdrawal-limit
        :param currency:  XBT  str (Mandatory)
        :return:
        {
          "currency": "XBT",//Currency
          "limitAmount": 2,//24h withdrawal limit
          "usedAmount": 0,//Withdrawal amount over the past 24h.
          "remainAmount": 2,//24h available withdrawal amount
          "availableAmount": 99.89993052,//Available balance
          "withdrawMinFee": 0.0005,//Withdrawal fee charges
          "innerWithdrawMinFee": 0,//Inner withdrawal fee charges
          "withdrawMinSize": 0.002,//Min. withdrawal amount
          "isWithdrawEnabled": true,//Available to withdrawal or not
          "precision": 8//Precision of the withdrawal amount
        }
        """
        params = {
            'currency': currency
        }
        return self._request('GET', '/api/v1/withdrawals/quotas', params=params)

    def sand_withdrawal(self, currency, address, amount, **kwargs):
        """
        https://docs.kumex.com/#withdraw-funds
        :param currency: Currency, only Bitcoin (XBT) is currently supported. (Mandatory)
        :type: str
        :param address: 	Withdrawal address (Mandatory)
        :type: str
        :param amount: Withdrawal amount (Mandatory)
        :type: float
        :param kwargs:  [Optional]  isInner, remark
        :return:
         {
            "withdrawalId": "" // Withdrawal ID. This ID can be used to cancel the withdrawal
        }
        """
        params = {
            'currency': currency,
            'address': address,
            'amount': amount
        }
        if kwargs:
            params.update(kwargs)

        return self._request('POST', '/api/v1/withdrawals', params=params)

    def get_withdrawal_list(self, **kwargs):
        """
        https://docs.kumex.com/#get-withdrawal-list
        :param kwargs: [optional] currentPage , pageSize  and so on
        :return:
         {
              "currentPage": 1,
              "pageSize": 50,
              "totalNum": 10,
              "totalPage": 1,
              "items": [{
                "withdrawalId": "5cda659603aa67131f305f7e",//Withdrawal ID. This ID can be used to cancel the withdrawal
                "currency": "XBT",//Currency
                "status": "FAILURE",//Status
                "address": "3JaG3ReoZCtLcqszxMEvktBn7xZdU9gaoJ",//Withdrawal address
                "isInner": true,//Inner withdrawal or not
                "amount": 1,//Withdrawal amount
                "fee": 0,//Withdrawal fee charges
                "walletTxId": "",//Wallet TXID
                "createdAt": 1557816726000,//Withdrawal time
                "remark": "",//Withdrawal remarks
                "reason": "Assets freezing failed."// Reason causing the failure
          }]
        }
        """
        params = {}
        if kwargs:
            params.update(kwargs)

        return self._request('GET', '/api/v1/withdrawal-list', params=params)

    def cancel_withdrawal(self, withdrawalId):
        """
        https://docs.kumex.com/#cancel-withdrawal
        :param withdrawalId: Path Parameter. Withdrawal ID  (Mandatory)
        :type: str
        :return: {'address': '', 'memo': ''}
        """
        return self._request('DELETE', '/api/v1/withdrawals/{withdrawalId}'.format(withdrawalId=withdrawalId))

    def get_deposit_address(self, currency):
        """
        https://docs.kumex.com/#get-deposit-address
        :param currency:  XBT  str (Mandatory)
        :return:
        """
        params = {
            'currency': currency
        }
        return self._request('GET', '/api/v1/deposit-address', params=params)

    def get_deposit_list(self, **kwargs):
        """
        https://docs.kumex.com/#get-deposits-list

        :param kwargs:  [optional]  currentPage , pageSize  and so on
        :return:
            {
              "currentPage": 1,
              "pageSize": 50,
              "totalNum": 1,
              "totalPage": 1,
              "items": [{
                "currency": "XBT",//Currency
                "status": "SUCCESS",//Status type: PROCESSING, WALLET_PROCESSING, SUCCESS, FAILURE
                "address": "5CD018972914B66104BF8842",//Deposit address
                "isInner": false,//Inner transfer or not
                "amount": 1,//Deposit amount
                "fee": 0,//Fees for deposit
                "walletTxId": "5CD018972914B66104BF8842",//Wallet TXID
                "createdAt": 1557141673000 //Funds deposit time
              }]
            }

        """
        params = {}
        if kwargs:
            params.update(kwargs)

        return self._request('GET', '/api/v1/deposit-list', params=params)

    def get_account_overview(self, currency='XBT'):
        """
        https://docs.kumex.com/#get-account-overview
        :return:
        {
          "accountEquity": 99.8999305281, //Account equity
          "unrealisedPNL": 0, //Unrealised profit and loss
          "marginBalance": 99.8999305281, //Margin balance
          "positionMargin": 0, //Position margin
          "orderMargin": 0, //Order margin
          "frozenFunds": 0, //Frozen funds for withdrawal and out-transfer
          "availableBalance": 99.8999305281 //Available balance
          "currency": "XBT" //currency code
        }
        """
        params = {
            'currency': currency
        }
        return self._request('GET', '/api/v1/account-overview', params=params)

    def get_transaction_history(self, **kwargs):
        """
        https://docs.kumex.com/#get-transaction-history
        :param kwargs: [optional]  startAt, endAt, type, offset maxCount
        :return:
         {
          "hasMore": false,//Whether there are more pages
          "dataList": [{
            "time": 1558596284040, //Event time
            "type": "RealisedPNL", //Type
            "amount": 0, //Transaction amount
            "fee": null,//Fees
            "accountEquity": 8060.7899305281, //Account equity
            "status": "Pending", //Status. If you have held a position in the current 8-hour settlement period.
            "remark": "XBTUSDM",//Ticker symbol of the contract
            "offset": -1 //Offset,
            "currency": "XBT"  //Currency
          },
          {
            "time": 1557997200000,
            "type": "RealisedPNL",
            "amount": -0.000017105,
            "fee": 0,
            "accountEquity": 8060.7899305281,
            "status": "Completed",//Status. Status. Funding period that has been settled.
         "remark": "XBTUSDM",
         "offset": 1,
         "currency": "XBT"  //Currency
         }]
        }
        """
        params = {}
        if kwargs:
            params.update(kwargs)

        return self._request('GET', '/api/v1/transaction-history', params=params)



import time
from datetime import datetime

import pandas as pd

from Utils import DataHelpers


def getBalanceOut(data, single=False, futures=False):
    if not single:
        outData = []
        if not futures:
            for asset in data:
                coinData = {'asset': asset['coin'], 'free': asset['free'], 'locked': asset['locked'],
                            'exchangeSpecific': asset}
                outData.append(coinData)
            return outData
        else:
            for key, value in data.items():
                coinData = {'asset': key, 'free': value['available_balance'], 'locked': value['used_margin'],
                            'exchangeSpecific': value}
                outData.append(coinData)
            return outData
    else:
        print('\n\n\n\n{}\n\n\n\n'.format(data))
        if not futures:
            outData = {'asset': data['coin'], 'free': data['free'], 'locked': data['locked'], 'exchangeSpecific': data}
            return outData
        else:
            outData = {}
            key = list(data.keys())[0]

            outData['asset'] = key
            outData['free'] = data[key]['available_balance']
            outData['locked'] = data[key]['used_margin']
            outData['exchangeSpecific'] = data[key]
            return outData


def getMyTradeHistoryOut(data, futures=False):
    outData = []
    if futures:
        for history in data:
            outData.append(
                {'symbol': history['symbol'], 'id': history['exec_id'], 'orderId': history['order_id'],
                 'orderListId': history['order_link_id'], 'price': history['price'],
                 'qty': history['order_qty'],
                 'quoteQty': str(float(history['price']) * float(history['order_qty'])),
                 'commission': None, 'commissionAsset': None, 'time': history['trade_time_ms'],
                 'isBuyer': None, 'isMaker': None, 'isBestMatch': None, 'exchangeSpecific': history}
            )
    else:
        for history in data:
            outData.append(
                {'symbol': history['symbol'], 'id': history['id'], 'orderId': history['orderId'],
                 'orderListId': -1, 'price': history['price'], 'qty': history['qty'],
                 'quoteQty': str(float(history['price']) * float(history['qty'])),
                 'commission': history['commission'], 'commissionAsset': history['commissionAsset'],
                 'time': history['time'], 'isBuyer': history['isBuyer'], 'isMaker': history['isMaker'],
                 'isBestMatch': None, 'exchangeSpecific': history}
            )
    return outData


def getRecentTradeHistoryOut(data, futures=False):
    outData = []
    if futures:
        for datum in data:
            outData.append({
                'id': datum['id'], 'price': datum['price'], 'qty': datum['qty'],
                'quoteQty': str(float(datum['qty'] * datum['price'])),
                'time': datum['trade_time_ms'], 'isBuyerMaker': None, 'isBestMatch': None, 'exchangeSpecific': datum
            })
    else:
        for datum in data:
            outData.append({
                'id': None, 'price': datum['price'], 'qty': datum['qty'],
                'quoteQty': str(float(datum['qty']) * float(datum['price'])),
                'time': datum['time'], 'isBuyerMaker': datum['isBuyerMaker'], 'isBestMatch': None,
                'exchangeSpecific': datum
            })
    return outData


def getMakeSpotOrderOut(data):
    return {
        'symbol': data['symbol'],
        'orderId': data['orderId'],
        'orderListId': -1,
        'clientOrderId': data['orderLinkId'],
        'transactTime': data['transactTime'],
        'price': data['price'],
        'origQty': data['origQty'],
        'executedQty': data['executedQty'],
        'cummulativeQuoteQty': None,
        'status': data['status'],
        'timeInForce': data['timeInForce'],
        'type': data['type'],
        'side': data['side'],
        'fills': None,
        'exchangeSpecific': data
    }


def getOrderOut(data, futures=False):
    if futures:
        pass
    else:
        return {
            'symbol': data['symbol'],
            'orderId': data['orderId'],
            'orderListId': -1,
            'clientOrderId': data['orderLinkId'],
            'price': data['price'],
            'origQty': data['origQty'],
            'executedQty': data['executedQty'],
            'cummulativeQuoteQty': data['cummulativeQuoteQty'],
            'status': data['status'],
            'timeInForce': data['timeInForce'],
            'type': data['type'],
            'side': data['side'],
            'stopPrice': data['stopPrice'],
            'icebergQty': data['icebergQty'],
            'time': data['time'],
            'updateTime': data['updateTime'],
            'isWorking': data['isWorking'],
            'origQuoteOrderQty': None,
            'exchangeSpecific': data
        }


def getOpenOrdersOut(data, futures=False):
    outData = []
    if futures:
        pass
    else:
        for datum in data:
            outData.append({
                'symbol': datum['symbol'],
                'orderId': datum['orderId'],
                'orderListId': None,
                'clientOrderId': datum['orderLinkId'],
                'price': datum['price'],
                'origQty': datum['origQty'],
                'executedQty': datum['executedQty'],
                'cummulativeQuoteQty': datum['cummulativeQuoteQty'],
                'status': datum['status'],
                'timeInForce': datum['timeInForce'],
                'type': datum['type'],
                'side': datum['side'],
                'stopPrice': datum['stopPrice'],
                'icebergQty': datum['icebergQty'],
                'time': datum['time'],
                'updateTime': datum['updateTime'],
                'isWorking': datum['isWorking'],
                'origQuoteOrderQty': None,
                'exchangeSpecific': datum
            })
    return outData


def futuresOrderOut(data, isConditional=False):
    if isConditional:
        return {
            'symbol': data['symbol'],
            'orderId': data['stop_order_id'],
            'clientOrderId': data['order_link_id'],
            'transactTime': time.mktime(
                datetime.strptime(data['created_time'], '%Y-%m-%dT%H:%M:%SZ').timetuple()),
            'price': data['price'],
            'origQty': data['qty'],
            'executedQty': 0.0,
            'cummulativeQuoteQty': 0.0,
            'status': data['order_status'],
            'timeInForce': data['time_in_force'],
            'type': data['order_type'],
            'side': data['side'],
            'extraData': {
                'reduceOnly': data['reduce_only'],
                'stopPrice': data['trigger_price'],
                'workingType': data['trigger_by'],
                'avgPrice': 0.0,
                'origType': data['order_type'],
                'positionSide': None,
                'activatePrice': None,
                'priceRate': None,
                'closePosition': data['close_on_trigger'],
            },
            'exchangeSpecific': data
        }
    else:
        return {
            'symbol': data['symbol'],
            'orderId': data['order_id'],
            'clientOrderId': data['order_link_id'],
            'transactTime': time.mktime(
                datetime.strptime(data['created_time'], '%Y-%m-%dT%H:%M:%SZ').timetuple()),
            'price': data['price'],
            'origQty': data['qty'],
            'executedQty': data['cum_exec_qty'],
            'cummulativeQuoteQty': data['cum_exec_value'],
            'status': data['order_status'],
            'timeInForce': data['time_in_force'],
            'type': data['order_type'],
            'side': data['side'],
            'extraData': {
                'reduceOnly': data['reduce_only'],
                'stopPrice': 0.0,
                'workingType': None,
                'avgPrice': 0.0,
                'origType': data['order_type'],
                'positionSide': None,
                'activatePrice': None,
                'priceRate': None,
                'closePosition': data['close_on_trigger'],
            },
            'exchangeSpecific': data
        }


def makeDummyBalance(asset):
    return {
        'asset': asset,
        'free': str(0.0),
        'locked': str(0.0),
        'exchangeSpecific': {}
    }


def getSpotOrderAsDict(order: DataHelpers.OrderData):
    params = {
        'symbol': order.symbol,
        'qty': order.quantity,
        'side': order.side,
        'type': order.orderType,
        'timeInForce': order.timeInForce,
        'price': order.price,
        'orderLinkId': order.newClientOrderId
    }
    return params


def getFuturesOrderAsDict(order: DataHelpers.futuresOrderData, timeInForceTranslate):
    if 'STOP' in order.orderType:
        params = {
            'side': order.side.lower().title(),
            'symbol': order.symbol,
            'order_type': 'Market' if order.orderType == 'STOP_MARKET' else 'Limit',
            'qty': order.quantity,
            'price': order.price,
            'base_price': order.extraParams['basePrice'],
            'stop_px': order.stopPrice,

            'time_in_force': order.timeInForce if order.timeInForce in timeInForceTranslate.values()
            else timeInForceTranslate[order.timeInForce],

            'close_on_trigger': order.closePosition,
            'reduce_only': order.reduceOnly
        }

        if 'triggerBy' in order.extraParams.keys():
            params['trigger_by'] = order.extraParams['triggerBy']

    else:
        params = {
            'side': order.side.lower().title(),
            'symbol': order.symbol,
            'order_type': order.orderType.lower().title(),
            'qty': order.quantity,

            'time_in_force': order.timeInForce if order.timeInForce in timeInForceTranslate.values()
            else timeInForceTranslate[order.timeInForce],

            'close_on_trigger': order.closePosition,
            'reduce_only': order.reduceOnly
        }

    if order.price is not None:
        params['price'] = order.price

    if order.newClientOrderId is not None:
        params['order_link_id'] = order.newClientOrderId

    if 'takeProfit' in order.extraParams.keys():
        params['take_profit'] = order.extraParams['takeProfit']

    if 'stopLoss' in order.extraParams.keys():
        params['stop_loss'] = order.extraParams['stopLoss']

    if 'tpTriggerBy' in order.extraParams.keys():
        params['tp_trigger_by'] = order.extraParams['tpTriggerBy']

    if 'slTriggerBy' in order.extraParams.keys():
        params['sl_trigger_by'] = order.extraParams['slTriggerBy']

    if 'positionIdx' in order.extraParams.keys():
        params['position_idx'] = order.extraParams['positionIdx']

    return params


def convertIntervalToFuturesKlines(interval):
    if interval == '1m':
        return 1
    elif interval == '3m':
        return 3
    elif interval == '5m':
        return 5
    elif interval == '15m':
        return 15
    elif interval == '30m':
        return 30
    elif interval == '1h':
        return 60
    elif interval == '2h':
        return 120
    elif interval == '4h':
        return 240
    elif interval == '6h':
        return 360
    elif interval == '12h':
        return 720
    elif interval == '1d':
        return 'D'
    elif interval == '1w':
        return 'W'
    elif interval == '1M':
        return 'M'


def getIntervalInSeconds(interval, validIntervals):
    if interval not in validIntervals:
        raise ValueError('Incorrect time interval specified')
    if interval == '1m':
        return 60
    elif interval == '3m':
        return 3 * 60
    elif interval == '5m':
        return 5 * 60
    elif interval == '15m':
        return 15 * 60
    elif interval == '30m':
        return 30 * 60
    elif interval == '1h':
        return 60 * 60
    elif interval == '2h':
        return 120 * 60
    elif interval == '4h':
        return 240 * 60
    elif interval == '6h':
        return 360 * 60
    elif interval == '12h':
        return 720 * 60
    elif interval == '1d':
        return 86400
    elif interval == '1w':
        return 7 * 86400
    elif interval == '1M':
        return 30 * 86400


def getKlinesDesiredOnlyCols(data, desiredIndexes):
    finalDataArray = []
    for datum in data:
        finalDataArray.append([datum[index] for index in desiredIndexes])
    return finalDataArray


def klinesConvertToPandas(finalDataArray):
    df = pd.DataFrame(finalDataArray,
                      columns=['date', 'open', 'high', 'low', 'close', 'volume', 'closeDate', 'tradesNum'])
    df.set_index('date', inplace=True)
    return df


def klinesConvertDate(data, futures, timeColIdxs):
    for datum in data:
        for idx in timeColIdxs:
            if futures:
                datum[idx] = datetime.fromtimestamp(float(datum[idx]))
            else:
                datum[idx] = datetime.fromtimestamp(float(datum[idx]) / 1000)



class TickerPriceChangeStatistics:

    def __init__(self):
        self.symbol = ""
        self.priceChange = 0.0
        self.priceChangePercent = 0.0
        self.weightedAvgPrice = 0.0
        self.lastPrice = 0.0
        self.lastQty = 0.0
        self.bidPrice = 0.0
        self.askPrice = 0.0
        self.openPrice = 0.0
        self.highPrice = 0.0
        self.lowPrice = 0.0
        self.volume = 0.0
        self.quoteVolume = 0.0
        self.openTime = 0
        self.closeTime = 0
        self.firstId = None
        self.lastId = None
        self.count = None

    @staticmethod
    def json_parse(json_data):
        result = TickerPriceChangeStatistics()
        result.symbol = json_data.get_string("symbol")
        result.priceChange = json_data.get_float("priceChange")
        result.priceChangePercent = json_data.get_float("priceChangePercent")
        result.weightedAvgPrice = json_data.get_float("weightedAvgPrice")
        result.lastPrice = json_data.get_float("lastPrice")
        result.lastQty = json_data.get_float("lastQty")
        result.openPrice = json_data.get_float("openPrice")
        result.highPrice = json_data.get_float("highPrice")
        result.lowPrice = json_data.get_float("lowPrice")
        result.volume = json_data.get_float("volume")
        result.quoteVolume = json_data.get_float("quoteVolume")
        result.openTime = json_data.get_int("openTime")
        result.closeTime = json_data.get_int("closeTime")
        result.firstId = json_data.get_int("firstId")
        result.lastId = json_data.get_int("lastId")
        result.count = json_data.get_int("count")
        return result


import json
import logging
import time

import pytest

from TradeGates.TradeGate import TradeGate

loglevel = logging.INFO
logging.basicConfig(level=loglevel)
log = logging.getLogger(__name__)


@pytest.fixture
def getGates():
    gates = []
    with open('./config.json') as f:
        config = json.load(f)

    for key in config.keys():
        gates.append(TradeGate(config[key], sandbox=True))

    return gates


def testNewTestOrder(getGates):
    for gate in getGates:
        try:
            res = gate.createAndTestSpotOrder('BTCUSDT', 'SELL', 'LIMIT', timeInForce='GTC', quantity=0.002,
                                              price=49500)
            assert res is not None, 'Problem in testing making order from {} exchange'.format(gate.exchangeName)
        except Exception as e:
            assert False, 'From {} exchange : {}'.format(gate.exchangeName, str(e))


def testNewTestOrderBadOrderType(getGates):
    for gate in getGates:
        try:
            res = gate.createAndTestSpotOrder('BTCUSDT', 'SELL', 'LINIT', timeInForce='GTC', quantity=0.002,
                                              price=49500)
            assert res is None, 'Bad order type and information provided. Must fail (Exchange: {})'.format(
                gate.exchangeName)
        except Exception:
            assert True, 'Bad order type and information provided. Must fail (Exchange: {})'.format(gate.exchangeName)


def testNewOrder(getGates):
    for gate in getGates:
        try:
            verifiedOrder = gate.createAndTestSpotOrder('BTCUSDT', 'BUY', 'LIMIT', quantity=0.002, price=35000,
                                                        timeInForce='GTC')
            result = gate.makeSpotOrder(verifiedOrder)
            assert result is not None, 'Problem in making new order in {} exchange'.format(gate.exchangeName)
        except Exception:
            assert False, 'Problem in making new order in {} exchange'.format(gate.exchangeName)


def testGetOrders(getGates):
    for gate in getGates:
        orders = gate.getSymbolOrders('BTCUSDT', futures=False)
        # print('\nGetting order history for BTCUSDT symbol from {}: {}'.format(gate.exchangeName, orders[0]))

        assert orders is not None, 'Problem in getting list of all orders from {} exchange.'.format(gate.exchangeName)


def testGetOpenOrders(getGates):
    for gate in getGates:
        symbolOpenOrders = gate.getOpenOrders('BTCUSDT')
        # print('\nGetting BTCUSDT open orders list from {} exchange: {}'.format(gate.exchangeName, symbolOpenOrders))
        assert symbolOpenOrders is not None, 'Problem in getting list of open orders with symbol from {} exchange.'.format(
            gate.exchangeName)


def testGetOrder(getGates):
    for gate in getGates:
        try:
            verifiedOrder = gate.createAndTestSpotOrder('BTCUSDT', 'BUY', 'LIMIT', quantity=0.02, price=34000,
                                                        timeInForce='GTC', newClientOrderId=str(int(time.time())))
            result = gate.makeSpotOrder(verifiedOrder)
        except Exception as e:
            assert False, 'Problem in making order from {} exchange: {}'.format(gate.exchangeName, str(e))

        # print('Submitted order on {} exchange: {}'.format(gate.exchangeName, result))

        order = gate.getOrder('BTCUSDT', orderId=result['orderId'])
        assert order['clientOrderId'] == result['clientOrderId'], \
            'Fetch client orderID is not equal to the actual client orderID from {} exchange.'.format(gate.exchangeName)
        # print('Correct \'clientOrderId\'.')

        order = gate.getOrder('BTCUSDT', localOrderId=result['clientOrderId'])
        assert order['orderId'] == result['orderId'], \
            'Fetch orderID is not equal to the actual orderID from {} exchange.'.format(gate.exchangeName)
        # print('Correct \'orderId\'.')

        gate.cancelOrder('BTCUSDT', orderId=result['orderId'])


def testCancelingAllOpenOrders(getGates):
    for gate in getGates:
        gate.cancelAllSymbolOpenOrders('BTCUSDT')

        openOrders = gate.getOpenOrders('BTCUSDT')
        assert len(openOrders) == 0, 'Problem in canceling all Open Orders in {} exchange.'.format(gate.exchangeName)


def testCancelingOrder(getGates):
    for gate in getGates:
        try:
            verifiedOrder = gate.createAndTestSpotOrder('BTCUSDT', 'BUY', 'LIMIT', quantity=0.002, price=35000,
                                                        timeInForce='GTC', newClientOrderId=str(int(time.time())))
            result = gate.makeSpotOrder(verifiedOrder)
        except Exception as e:
            assert False, 'Problem in making order in {} exchange: {}'.format(gate.exchangeName, str(e))

        result = gate.cancelOrder(symbol='BTCUSDT', orderId=result['orderId'])
        result = gate.getOrder(symbol='BTCUSDT', orderId=result['orderId'])

        assert result['status'] == 'CANCELED', 'Problem in canceling specified Open Orders in {} exchange.'.format(
            gate.exchangeName)

        try:
            verifiedOrder = gate.createAndTestSpotOrder('BTCUSDT', 'BUY', 'LIMIT', quantity=0.002, price=35000,
                                                        timeInForce='GTC', newClientOrderId=str(int(time.time())))
            result = gate.makeSpotOrder(verifiedOrder)
        except Exception as e:
            assert False, 'Problem in making order in {} exchange: {}'.format(gate.exchangeName, str(e))

        result = gate.cancelOrder(symbol='BTCUSDT', localOrderId=result['clientOrderId'])
        result = gate.getOrder(symbol='BTCUSDT', orderId=result['orderId'])

        assert result['status'] == 'CANCELED', 'Problem in canceling specified Open Orders in {} exchange.'.format(
            gate.exchangeName)



import os
if(os.path.exists("binance_f/privateconfig.py")):
    from binance_f.privateconfig import *
    g_api_key = p_api_key
    g_secret_key = p_secret_key
else:
    g_api_key = ""
    g_secret_key = ""


g_account_id = 12345678






from kucoin.margin.margin import MarginData
from kucoin.market.market import MarketData
from kucoin.trade.trade import TradeData
from kucoin.user.user import UserData
from kucoin.ws_token.token import GetToken


class User(UserData):
    pass


class Trade(TradeData):
    pass


class Market(MarketData):
    pass


class Margin(MarginData):
    pass


class WsToken(GetToken):
    pass





import time


class OrderData():
    def __init__(self, symbol, side, orderType):
        self.symbol = symbol
        self.side = side
        self.orderType = orderType

        self.timeInForce = None
        self.quantity = None
        self.quoteOrderQty = None
        self.price = None
        self.newClientOrderId = None
        self.stopPrice = None
        self.icebergQty = None
        self.newOrderRespType = None
        self.recvWindow = None
        self.timestamp = None

    def setTimeInForce(self, timeInForce):
        self.timeInForce = timeInForce

    def setQuantity(self, quantity):
        self.quantity = quantity

    def setQuoteOrderQty(self, quoteOrderQty):
        self.quoteOrderQty = quoteOrderQty

    def setPrice(self, price):
        self.price = price

    def setNewClientOrderId(self, newClientOrderId):
        self.newClientOrderId = newClientOrderId

    def setStopPrice(self, stopPrice):
        self.stopPrice = stopPrice

    def setIcebergQty(self, icebergQty):
        self.icebergQty = icebergQty

    def setNewOrderRespType(self, newOrderRespType):
        self.newOrderRespType = newOrderRespType

    def setRecvWindow(self, recvWindow):
        self.recvWindow = recvWindow

    def setTimestamp(self):
        self.timestamp = time.time()


class futuresOrderData():
    def __init__(self, symbol, side, orderType):
        self.symbol = symbol
        self.side = side
        self.orderType = orderType

        self.positionSide = None
        self.timeInForce = None
        self.quantity = None
        self.reduceOnly = None
        self.price = None
        self.newClientOrderId = None
        self.stopPrice = None
        self.closePosition = None
        self.activationPrice = None
        self.callbackRate = None
        self.workingType = None
        self.priceProtect = None
        self.newOrderRespType = None
        self.recvWindow = None
        self.extraParams = None

    def setPositionSide(self, positionSide):
        self.positionSide = positionSide

    def setTimeInForce(self, timeInForce):
        self.timeInForce = timeInForce

    def setQuantity(self, quantity):
        self.quantity = quantity

    def setReduceOnly(self, reduceOnly):
        self.reduceOnly = reduceOnly

    def setPrice(self, price):
        self.price = price

    def setNewClientOrderId(self, newClientOrderId):
        self.newClientOrderId = newClientOrderId

    def setStopPrice(self, stopPrice):
        self.stopPrice = stopPrice

    def setClosePosition(self, closePosition):
        self.closePosition = closePosition

    def setActivationPrice(self, activationPrice):
        self.activationPrice = activationPrice

    def setCallbackRate(self, callbackRate):
        self.callbackRate = callbackRate

    def setWorkingType(self, workingType):
        self.workingType = workingType

    def setPriceProtect(self, priceProtect):
        self.priceProtect = priceProtect

    def setNewOrderRespType(self, newOrderRespType):
        self.newOrderRespType = newOrderRespType

    def setRecvWindow(self, recvWindow):
        self.recvWindow = recvWindow

    def setExtraParams(self, extraParams):
        self.extraParams = extraParams



class Order:

    def __init__(self):
        self.clientOrderId = ""
        self.cumQuote = 0.0
        self.executedQty = None
        self.orderId = None
        self.origQty = None
        self.price = None
        self.reduceOnly = False
        self.side = None
        self.status = None
        self.stopPrice = None
        self.symbol = ""
        self.timeInForce = None
        self.type = None
        self.updateTime = 0
        self.workingType = ""
        self.avgPrice = 0.0
        self.origType = ""
        self.positionSide = ""
        self.activatePrice = None
        self.priceRate = None
        self.closePosition = None

    @staticmethod
    def json_parse(json_data):
        result = Order()
        result.clientOrderId = json_data.get_string("clientOrderId")
        result.cumQuote = json_data.get_float("cumQuote")
        result.executedQty = json_data.get_float_or_default("executedQty", None)
        result.orderId = json_data.get_int("orderId")
        result.origQty = json_data.get_float("origQty")
        result.price = json_data.get_float("price")
        result.reduceOnly = json_data.get_boolean("reduceOnly")
        result.side = json_data.get_string("side")
        result.status = json_data.get_string("status")
        result.stopPrice = json_data.get_float("stopPrice")
        result.symbol = json_data.get_string("symbol")
        result.timeInForce = json_data.get_string("timeInForce")
        result.type = json_data.get_string("type")
        result.updateTime = json_data.get_int("updateTime")
        result.workingType = json_data.get_string("workingType")
        result.avgPrice = json_data.get_float("avgPrice")
        result.origType = json_data.get_string("origType")
        result.positionSide = json_data.get_string("positionSide")
        result.activatePrice = json_data.get_float_or_default("activatePrice", None)
        result.priceRate = json_data.get_float_or_default("priceRate", None)
        result.closePosition = json_data.get_boolean("closePosition")

        return result

    def toDict(self):
        outDict = {
            'symbol': self.symbol,
            'orderId': self.orderId,
            'clientOrderId': self.clientOrderId,
            'transactTime': self.updateTime,
            'price': self.price,
            'origQty': self.origQty,
            'executedQty': self.executedQty,
            'cummulativeQuoteQty': self.cumQuote,
            'status': self.status,
            'timeInForce': self.timeInForce,
            'type': self.type,
            'side': self.side,
        }

        extraData = {
            'reduceOnly': self.reduceOnly,
            'stopPrice': self.stopPrice,
            'workingType': self.workingType,
            'avgPrice': self.avgPrice,
            'origType': self.origType,
            'positionSide': self.positionSide,
            'activatePrice': self.activatePrice,
            'priceRate': self.priceRate,
            'closePosition': self.closePosition
        }

        outDict['extraData'] = extraData

        return outDict



class Asset:

    def __init__(self):
        self.asset = ""
        self.initialMargin = 0.0
        self.maintMargin = 0.0
        self.marginBalance = 0.0
        self.maxWithdrawAmount = 0.0
        self.openOrderInitialMargin = 0.0
        self.positionInitialMargin = 0.0
        self.unrealizedProfit = 0.0
        self.walletBalance = 0.0

    @staticmethod
    def json_parse(json_data):
        result = Asset()
        result.asset = json_data.get_string("asset")
        result.initialMargin = json_data.get_float("initialMargin")
        result.maintMargin = json_data.get_float("maintMargin")
        result.marginBalance = json_data.get_float("marginBalance")
        result.maxWithdrawAmount = json_data.get_float("maxWithdrawAmount")
        result.openOrderInitialMargin = json_data.get_float("openOrderInitialMargin")
        result.positionInitialMargin = json_data.get_float("positionInitialMargin")
        result.unrealizedProfit = json_data.get_float("unrealizedProfit")
        return result


class Position:

    def __init__(self):
        self.initialMargin = 0.0
        self.maintMargin = 0.0
        self.openOrderInitialMargin = 0.0
        self.positionInitialMargin = 0.0
        self.symbol = ""
        self.leverage = 0.0
        self.unrealizedProfit = 0.0
        self.isolated = False
        self.positionSide = ""

    @staticmethod
    def json_parse(json_data):
        result = Position()
        result.initialMargin = json_data.get_float("initialMargin")
        result.maintMargin = json_data.get_float("maintMargin")
        result.leverage = json_data.get_float("leverage")
        result.openOrderInitialMargin = json_data.get_float("openOrderInitialMargin")
        result.positionInitialMargin = json_data.get_float("positionInitialMargin")
        result.symbol = json_data.get_string("symbol")
        result.unrealizedProfit = json_data.get_float("unrealizedProfit")
        result.isolated = json_data.get_boolean("isolated")
        result.positionSide = json_data.get_string("positionSide")
        return result


class AccountInformation:
    def __init__(self):
        self.canDeposit = False
        self.canTrade = False
        self.canWithdraw = False
        self.feeTier = 0
        self.maxWithdrawAmount = 0.0
        self.totalInitialMargin = 0.0
        self.totalMaintMargin = 0.0
        self.totalMarginBalance = 0.0
        self.totalOpenOrderInitialMargin = 0.0
        self.totalPositionInitialMargin = 0.0
        self.totalUnrealizedProfit = 0.0
        self.totalWalletBalance = 0.0
        self.updateTime = 0
        self.assets = list()
        self.positions = list()

    @staticmethod
    def json_parse(json_data):
        result = AccountInformation()
        result.canDeposit = json_data.get_boolean("canDeposit")
        result.canTrade = json_data.get_boolean("canTrade")
        result.canWithdraw = json_data.get_boolean("canWithdraw")
        result.feeTier = json_data.get_float("feeTier")
        result.maxWithdrawAmount = json_data.get_float("maxWithdrawAmount")
        result.totalInitialMargin = json_data.get_float("totalInitialMargin")
        result.totalMaintMargin = json_data.get_float("totalMaintMargin")
        result.totalMarginBalance = json_data.get_float("totalMarginBalance")
        result.totalOpenOrderInitialMargin = json_data.get_float("totalOpenOrderInitialMargin")
        result.totalPositionInitialMargin = json_data.get_float("totalPositionInitialMargin")
        result.totalUnrealizedProfit = json_data.get_float("totalUnrealizedProfit")
        result.totalWalletBalance = json_data.get_float("totalWalletBalance")
        result.updateTime = json_data.get_int("updateTime")
        
        element_list = list()
        data_list = json_data.get_array("assets")
        for item in data_list.get_items():
            element = Asset.json_parse(item)
            element_list.append(element)
        result.assets = element_list
        
        element_list = list()
        data_list = json_data.get_array("positions")
        for item in data_list.get_items():
            element = Position.json_parse(item)
            element_list.append(element)
        result.positions = element_list

        return result



class BLVTInfoEvent:

    def __init__(self):
        self.eventType = ""
        self.eventTime = 0
        self.symbol = ""
        self.circulation = 0.0
        self.baskets = 0.0
        self.NAV = 0.0
        self.lev = 0.0  # Real Leverage
        self.target = 0.0  #  Target Leverage
        self.fundingRate = 0.0

    @staticmethod
    def json_parse(json_data):
        result = BLVTInfoEvent()
        result.eventType = json_data.get_string("e")
        result.eventTime = json_data.get_int("E")
        result.symbol = json_data.get_string("s")
        result.circulation = json_data.get_float("m")
        result.baskets = json_data.get_string("b")
        result.NAV = json_data.get_float("n")
        result.lev = json_data.get_float("l")
        result.target = json_data.get_float("t")
        result.fundingRate = json_data.get_float("f")
        return result





class MarkPrice:

    def __init__(self):
        self.symbol = ""
        self.markPrice = 0.0
        self.lastFundingRate = 0.0
        self.nextFundingTime = 0
        self.time = 0
    
    @staticmethod
    def json_parse(json_data):
        result = MarkPrice()
        result.symbol = json_data.get_string("symbol")
        result.markPrice = json_data.get_float("markPrice")
        result.lastFundingRate = json_data.get_float("lastFundingRate")
        result.nextFundingTime = json_data.get_int("nextFundingTime")
        result.time = json_data.get_int("time")

        return result



class BalanceV2:

    def __init__(self):
        self.accountAlias = ""
        self.asset = ""
        self.balance = 0.0
        self.crossWalletBalance = 0.0
        self.crossUnPnl = 0.0
        self.availableBalance = 0.0
        self.maxWithdrawAmount = 0.0

    @staticmethod
    def json_parse(json_data):
        result = BalanceV2()
        result.accountAlias = json_data.get_string("accountAlias")
        result.asset = json_data.get_string("asset")
        result.balance = json_data.get_float("balance")
        result.crossWalletBalance = json_data.get_float("crossWalletBalance")
        result.crossUnPnl = json_data.get_float("crossUnPnl")
        result.availableBalance = json_data.get_float("availableBalance")
        result.maxWithdrawAmount = json_data.get_float("maxWithdrawAmount")

        return result




class OpenInterestStats:

    def __init__(self):
        self.symbol = ""
        self.sumOpenInterest = 0.0  #  total open interest
        self.sumOpenInterestValue = 0.0  # total open interest value
        self.timestamp = 0
    
    @staticmethod
    def json_parse(json_data):
        result = OpenInterestStats()
        result.symbol = json_data.get_string("symbol")
        result.sumOpenInterest = json_data.get_float("sumOpenInterest")
        result.sumOpenInterestValue = json_data.get_float("sumOpenInterestValue")
        result.timestamp = json_data.get_string("timestamp")

        return result



class Balance:

    def __init__(self):
        self.asset = ""
        self.accountAlias = ""
        self.balance = 0.0
        self.withdrawAvailable = 0.0

    @staticmethod
    def json_parse(json_data):
        result = Balance()
        result.asset = json_data.get_string("asset")
        result.accountAlias = json_data.get_string("accountAlias")
        result.balance = json_data.get_float("balance")
        result.withdrawAvailable = json_data.get_float("withdrawAvailable")

        return result

    @staticmethod
    def makeFreeBalance(asset):
        outDict = {}
        
        outDict['asset'] = asset
        outDict['free'] = str(0.0)
        outDict['locked'] = str(0.0)

        return outDict


    def toDict(self):
        outDict = {}
        
        outDict['asset'] = self.asset
        outDict['free'] = str(self.withdrawAvailable)
        outDict['locked'] = str(self.balance - self.withdrawAvailable)

        return outDict



import json
import logging
import time

import pytest

from TradeGates.TradeGate import TradeGate

loglevel = logging.INFO
logging.basicConfig(level=loglevel)
log = logging.getLogger(__name__)


@pytest.fixture
def getGates():
    gates = []
    with open('./config.json') as f:
        config = json.load(f)

    for key in config.keys():
        gates.append(TradeGate(config[key], sandbox=True))

    return gates


def testSymbolFuturesOrders(getGates):
    for gate in getGates:
        symbolFutureOrders = gate.getSymbolOrders('BTCUSDT', futures=True)
        # print('\nSymbol future orders from {} exchange: {}'.format(gate.exchangeName, symbolFutureOrders))
        assert symbolFutureOrders is not None, 'Problem in futures order list from {} exchange.'.format(
            gate.exchangeName)


def testFuturesBalance(getGates):
    for gate in getGates:
        balance = gate.getBalance(futures=True)
        # print('\nFutures balance from {} exchange: {}'.format(gate.exchangeName, balance))
        assert balance is not None, 'Problem in futures balance from {} exchange.'.format(gate.exchangeName)

        try:
            if not gate.exchangeName == 'Binance':
                if not sorted(list(balance[0].keys())) == sorted(['asset', 'free', 'locked', 'exchangeSpecific']):
                    assert False, 'Bad fetch balance interface for {} exchange,'.format(gate.exchangeName)
            else:
                if not sorted(list(balance[0].keys())) == sorted(['asset', 'free', 'locked']):
                    assert False, 'Bad fetch balance interface for {} exchange,'.format(gate.exchangeName)
        except:
            assert False, 'Bad fetch single coin balance interface for {} exchange,'.format(gate.exchangeName)


def testFuturesSingleCoinBalance(getGates):
    for gate in getGates:
        balance = gate.getBalance('USDT', futures=True)
        # print('\nUSDT Futures balance from {} exchange: {}'.format(gate.exchangeName, balance))
        assert balance is not None, 'Problem in fetching futures single coin balance from {} exchange.'.format(
            gate.exchangeName)

        try:
            if not gate.exchangeName == 'Binance':
                if not sorted(list(balance.keys())) == sorted(['asset', 'free', 'locked', 'exchangeSpecific']):
                    assert False, 'Bad fetch balance interface for {} exchange,'.format(gate.exchangeName)
            else:
                if not sorted(list(balance.keys())) == sorted(['asset', 'free', 'locked']):
                    assert False, 'Bad fetch balance interface for {} exchange,'.format(gate.exchangeName)
        except:
            assert False, 'Bad fetch single coin balance interface for {} exchange,'.format(gate.exchangeName)


def testCreatingFuturesOrder(getGates):
    for gate in getGates:
        futuresOrderData = gate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'MARKET', quantity=0.002)
        # print('\nTest creating futures order in {} exchange: {}'.format(gate.exchangeName, futuresOrderData))
        assert futuresOrderData is not None, 'Problem in creating futures order in {} exchange.'.format(
            gate.exchangeName)


def testFuturesOrder(getGates):
    for gate in getGates:
        futuresOrderData = gate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'MARKET', quantity=0.002)
        result = gate.makeFuturesOrder(futuresOrderData)
        # print('\nFuture ordering in {} exchange: {}'.format(gate.exchangeName, result))
        assert result is not None, 'Problem in submiting futures order in {} exchange.'.format(gate.exchangeName)


def testBatchFuturesOrders(getGates):
    for gate in getGates:
        try:
            verifiedOrders = [gate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'MARKET', quantity=0.1),
                              gate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'MARKET', quantity=0.2),
                              gate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'MARKET', quantity=0.3)]

            result = gate.makeBatchFuturesOrder(verifiedOrders)
            # print('\nResult of batch ordering from {} exchange: {}'.format(gate.exchangeName, result))
            assert result is not None, 'Problem in making new order in {} exchange'.format(gate.exchangeName)
        except Exception:
            assert False, 'Problem in making new order in {} exchange'.format(gate.exchangeName)


# @pytest.mark.skip(reason="For Special Purposes")
def testFuturesTpSlLimitOrder(getGates):
    for gate in getGates:
        if gate.exchangeName.lower() != 'binance':
            continue
        try:
            result = gate.makeSlTpLimitFuturesOrder(symbol='ADAUSDT', orderSide='BUY', quantity=None, quoteQuantity=40,
                                                    enterPrice=1.1649, takeProfit=1.1473, stopLoss=1.1974,
                                                    leverage=10,
                                                    marginType='ISOLATED')
            # print('\nResult of TP-SL-Limit ordering from {} exchange: {}'.format(gate.exchangeName, result))
            assert result is not None, 'Problem in making new order in {} exchange'.format(gate.exchangeName)
        except Exception:
            assert False, 'Problem in making new SL-TP-Limit order in {} exchange'.format(gate.exchangeName)


def testGetFuturesOpenOrders(getGates):
    for gate in getGates:
        symbolOpenOrders = gate.getOpenOrders('BTCUSDT', futures=True)

        # print('\n\'BTCUSDT\' open orders from {} exchange: {}'.format(gate.exchangeName, symbolOpenOrders))

        assert symbolOpenOrders is not None, \
            'Problem in getting list of open orders with symbol from {} exchange.'.format(gate.exchangeName)


def testGetPositionInformation(getGates):
    for gate in getGates:
        openPosition = gate.getPositionInfo('BTCUSDT')

        # print('\nOpen position information from {} exchange: {}'.format(gate.exchangeName, openPosition))

        assert openPosition is not None, 'Problem in getting position information without symbol from {} exchange.'.format(
            gate.exchangeName)


def testGetFutureOrder(getGates):
    for gate in getGates:
        futuresOrderData = gate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'LIMIT', quantity=0.002, price=40000,
                                                          timeInForce='GTC', newClientOrderId=str(int(time.time())))
        result = gate.makeFuturesOrder(futuresOrderData)
        order = gate.getOrder('BTCUSDT', orderId=result['orderId'], futures=True)

        # print('\nOrder data fetched from {} exchange: {}'.format(gate.exchangeName, order))

        assert order['clientOrderId'] == result[
            'clientOrderId'], 'Futures fetch client orderID is not equal to the actual client orderID from {} exchange.'.format(
            gate.exchangeName)

        order = gate.getOrder('BTCUSDT', localOrderId=result['clientOrderId'], futures=True)
        assert order['orderId'] == result[
            'orderId'], 'Futures fetch orderID is not equal to the actual orderID from {} exchange.'.format(
            gate.exchangeName)


def testCancelingAllFuturesOpenOrders(getGates):
    for gate in getGates:
        futuresOrderData = gate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'LIMIT', price=35000, quantity=0.002,
                                                          timeInForce='GTC')
        gate.makeFuturesOrder(futuresOrderData)

        gate.cancelAllSymbolOpenOrders('BTCUSDT', futures=True)

        openOrders = gate.getOpenOrders('BTCUSDT', futures=True)
        assert len(openOrders) == 0, 'Problem in canceling all Open Orders in {} exchange.'.format(gate.exchangeName)


def testCancelingOrder(getGates):
    for gate in getGates:
        futuresOrderData = gate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'LIMIT', price=35000, quantity=0.002,
                                                          timeInForce='GTC', newClientOrderId=str(int(time.time())))
        result = gate.makeFuturesOrder(futuresOrderData)

        gate.cancelOrder(symbol='BTCUSDT', localOrderId=result['clientOrderId'], futures=True)

        result = gate.getOrder(symbol='BTCUSDT', localOrderId=result['clientOrderId'], futures=True)
        assert result['status'] in ['CANCELED', 'Cancelled'], \
            'Problem in canceling specified Open Orders from {} exchange.'.format(gate.exchangeName)



import time
from binance_f.impl.websocketrequest import WebsocketRequest
from binance_f.impl.utils.channels import *
from binance_f.impl.utils.channelparser import ChannelParser
from binance_f.impl.utils.timeservice import *
from binance_f.impl.utils.inputchecker import *
from binance_f.model import *
# For develop
from binance_f.base.printobject import *



class WebsocketRequestImpl(object):

    def __init__(self, api_key):
        self.__api_key = api_key

    def subscribe_aggregate_trade_event(self, symbol, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(aggregate_trade_channel(symbol))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = AggregateTradeEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_mark_price_event(self, symbol, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(mark_price_channel(symbol))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = MarkPriceEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_continuous_candlestick_event(self, pair, contract_type, interval, callback, error_handler=None):
        check_should_not_none(pair, "pair")
        check_should_not_none(contract_type, "contract_type")
        check_should_not_none(interval, "interval")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(continuous_kline_channel(pair, contract_type, interval))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = ContinuousCandlestickEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request


    
    def subscribe_candlestick_event(self, symbol, interval, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(interval, "interval")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(kline_channel(symbol, interval))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = CandlestickEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request
    
    def subscribe_symbol_miniticker_event(self, symbol, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(symbol_miniticker_channel(symbol))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = SymbolMiniTickerEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_all_miniticker_event(self, callback, error_handler=None):
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(all_miniticker_channel())
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = list()
            data_list = json_wrapper.convert_2_array()
            for item in data_list.get_items():
                element = SymbolMiniTickerEvent.json_parse(item)
            result.append(element)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_symbol_ticker_event(self, symbol, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(symbol_ticker_channel(symbol))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = SymbolTickerEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request
    
    def subscribe_all_ticker_event(self, callback, error_handler=None):
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(all_ticker_channel())
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = list()
            data_list = json_wrapper.convert_2_array()
            for item in data_list.get_items():
                ticker_event_obj = SymbolTickerEvent.json_parse(item)
            result.append(ticker_event_obj)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_symbol_bookticker_event(self, symbol, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(symbol_bookticker_channel(symbol))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = SymbolBookTickerEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_all_bookticker_event(self, callback, error_handler=None):
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(all_bookticker_channel())
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = SymbolBookTickerEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request
    
    def subscribe_symbol_liquidation_event(self, symbol, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(symbol_liquidation_channel(symbol))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = LiquidationOrderEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request
    
    def subscribe_all_liquidation_event(self, callback, error_handler=None):
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(all_liquidation_channel())
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = LiquidationOrderEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_book_depth_event(self, symbol, limit, update_time, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(limit, "limit")
        check_should_not_none(callback, "callback")
        #print(update_time)
        def subscription_handler(connection):
            connection.send(book_depth_channel(symbol, limit, update_time))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = OrderBookEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_diff_depth_event(self, symbol, update_time, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(diff_depth_channel(symbol, update_time))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = DiffDepthEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

  
    def subscribe_user_data_event(self, listenKey, callback, error_handler=None):
        check_should_not_none(listenKey, "listenKey")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(user_data_channel(listenKey))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            # print("event type: ", json_wrapper.get_string("e"))
            # print(json_wrapper)
            if(json_wrapper.get_string("e") == "ACCOUNT_UPDATE"):
                result = AccountUpdate.json_parse(json_wrapper)
            elif(json_wrapper.get_string("e") == "ORDER_TRADE_UPDATE"):
                result = OrderUpdate.json_parse(json_wrapper)
            elif(json_wrapper.get_string("e") == "listenKeyExpired"):
                result = ListenKeyExpired.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request
           
 
    def subscribe_all_mark_price_event(self, callback, error_handler=None):
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(all_mark_price_channel())
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = list()
            data_list = json_wrapper.convert_2_array()
            for item in data_list.get_items():
                element = MarkPriceEvent.json_parse(item)
            result.append(element)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request


    def subscribe_blvt_info_event(self, symbol, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(blvt_info_channel(symbol))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = BLVTInfoEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_blvt_nav_candlestick_event(self, symbol, interval, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(interval, "interval")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(blvt_nav_kline_channel(symbol, interval))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = BLVTNAVCandlestickEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request

    def subscribe_composite_index_event(self, symbol, callback, error_handler=None):
        check_should_not_none(symbol, "symbol")
        check_should_not_none(callback, "callback")

        def subscription_handler(connection):
            connection.send(composite_index_channel(symbol))
            time.sleep(0.01)

        def json_parse(json_wrapper):
            result = CompositeIndexEvent.json_parse(json_wrapper)
            return result

        request = WebsocketRequest()
        request.subscription_handler = subscription_handler
        request.json_parser = json_parse
        request.update_callback = callback
        request.error_handler = error_handler

        return request



class SymbolBookTickerEvent:

    def __init__(self):
        self.orderBookUpdateId = None
        self.symbol = ""
        self.bestBidPrice = 0.0
        self.bestBidQty = 0.0
        self.bestAskPrice = 0.0
        self.bestAskQty = 0.0

    @staticmethod
    def json_parse(json_wrapper):
        ticker_event = SymbolBookTickerEvent()
        ticker_event.orderBookUpdateId = json_wrapper.get_int("u")
        ticker_event.symbol = json_wrapper.get_string("s")
        ticker_event.bestBidPrice = json_wrapper.get_float("b")
        ticker_event.bestBidQty = json_wrapper.get_float("B")
        ticker_event.bestAskPrice = json_wrapper.get_float("a")
        ticker_event.bestAskQty = json_wrapper.get_float("A")
        return ticker_event